/* -*- mode: C -*-  */
/* 
   IGraph library.
   Copyright (C) 2007  Gabor Csardi <csardi@rmki.kfki.hu>
   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc.,  51 Franklin Street, Fifth Floor, Boston, MA 
   02110-1301 USA

*/

#define _GNU_SOURCE

#include <libgen.h>
#include <string.h>
#include <stdio.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <limits.h>
#include <ctype.h>

#include "igraph.h"

/* ------------------------------------------------------------------------ */
/* Conversion & other common functions first                                */
/* ------------------------------------------------------------------------ */


int shell_igraph_usage(int argc, char **argv) {
  printf("Command line interface to igraph\n");
  return 0;
}

int shell_skip_whitespace(FILE *fin) {
  int c;
  c=fgetc(fin);
  while (isspace(c)) {
    c=fgetc(fin);
  }
  if (c!=EOF) { ungetc(c, fin); }
  return 0;
}

FILE *shell_open_file(const char *where, const char *mode) {  
  static const char *input="input";
  static const char *output="output";
  FILE *f;

  if (!strcmp(where, "-")) {
    if (!strcmp(mode, "r")) {
      return stdin;
    } else if (!strcmp(mode, "w")) {
      return stdout;
    }
  }

  f=fopen(where, mode);
  if (!f) {
    fprintf(stderr, "Cannot open %s file: `%s'\n", 
	    !strcmp(mode, "r") ? input : output, where);
    exit(1);
  }
  return f;
} 

int shell_read_graph(igraph_t *graph, const char *where) {
  FILE *fin=shell_open_file(where, "r");
  igraph_read_graph_graphml(graph, fin, 0);
  fclose(fin);
  return 0;
}

int shell_write_graph(const igraph_t *graph, const char *where) {
  FILE *fout=shell_open_file(where, "w");
  igraph_write_graph_graphml(graph, fout);
  fclose(fout);
  return 0;
}

int shell_read_vector(igraph_vector_t *v, const char *where) {
  FILE *fin=shell_open_file(where, "r");
  igraph_real_t n;
  int ret=1;
  igraph_vector_init(v, 0);
  while (ret > 0) {
    ret=fscanf(fin, "%lf", &n);
    if (ret > 0) { 
      igraph_vector_push_back(v, n);
    }
  }
  fclose(fin);
  if (ret != EOF) {
    fprintf(stderr, "Error reading vector from file: `%s'\n", where);
    exit(1);
  }

  return 0;
}

int shell_write_a_vector(const igraph_vector_t *v, FILE *fout, const char *where) {
  int ret=1;
  long int i, n=igraph_vector_size(v);
  if (n>0) { ret=fprintf(fout, "%g", VECTOR(*v)[0]); }
  if (ret <= 0) { 
    fprintf(stderr, "Cannot write vector to `%s'\n", where);
    fclose(fout);
    exit(1);
  }
  for (i=1; i<n; i++) {
    ret=fprintf(fout, " %g", VECTOR(*v)[i]);
    if (ret <= 0) { 
      fprintf(stderr, "Cannot write vector to `%s'\n", where);
      fclose(fout);
      exit(1);
    }
  }
  fprintf(fout, "\n");
  return 0;
}  

int shell_write_vector(const igraph_vector_t *v, const char *where) {
  FILE *fout=shell_open_file(where, "w");
  shell_write_a_vector(v, fout, where);
  fclose(fout);  
  return 0;
}

int shell_read_a_matrix(igraph_matrix_t *m, FILE *fin, const char *where) {
  long int ncol, nrow, i, j;
  igraph_real_t n;
  int ret;
  if (2 != fscanf(fin, "%li %li", &nrow, &ncol)) {
    fprintf(stderr, "Error reading matrix from file '%s'\n", where);
    fclose(fin);
    exit(1);
  }
  igraph_matrix_init(m,nrow,ncol);
  for (i=0; i<nrow; i++) {
    for (j=0; j<ncol; j++) {
      ret=fscanf(fin, "%lf", &n);
      if (ret <= 0) { 
	fprintf(stderr, "Error reading matrix from file '%s'\n", where);
	fclose(fin);
	exit(1);
      }
      MATRIX(*m, i, j)=n;
    }
  }
  return 0;
}

int shell_read_matrix(igraph_matrix_t *m, const char *where) {
  FILE *fin=shell_open_file(where, "r");
  shell_read_a_matrix(m, fin, where);
  fclose(fin);      
  return 0;
}

int shell_write_matrix(const igraph_matrix_t *m, const char *where) {
  FILE *fout=shell_open_file(where, "w");
  long int nrow=igraph_matrix_nrow(m);
  long int ncol=igraph_matrix_ncol(m);
  long int i, j;
    
  if (0>fprintf(fout, "%li %li\n", nrow, ncol)) {
    fprintf(stderr, "Error writing matrix to file '%s'.\n", where);
    fclose(fout);
    exit(1);
  }

  for (i=0; i<nrow; i++) {
    for (j=0; j<ncol; j++) {
      if (j!=0) { fprintf(fout, " "); }
      if (0>fprintf(fout, "%g", MATRIX(*m, i, j))) {
	fprintf(stderr, "Error writing matrix to file '%s'.\n", where);
	fclose(fout);
	exit(1);
      }
    }
    fprintf(fout, "\n");
  }
  fclose(fout);
  
  return 0;
}

int shell_read_integer(igraph_integer_t *n, const char *where) {
  long int nn;
  int ret=sscanf(where, "%li", &nn);
  if (ret == EOF || ret == 0) {
    fprintf(stderr, "Error, cannot interpret '%s' as integer\n", where);
    exit(1);
  }
  *n=nn;
  return 0;
}

int shell_write_integer(igraph_integer_t n, const char *where) {
  FILE *fout=shell_open_file(where, "w");
  int ret=fprintf(fout, "%li\n", (long int)n);
  if (ret <= 0) { 
    fprintf(stderr, "Cannot write integer to '%s'\n", where);
    fclose(fout);
    exit(1);
  }
  return 0;
}

int shell_read_boolean(igraph_bool_t *b, const char *where) {
  if (strlen(where)==0 || 
      where[0]=='0' ||
      where[0]=='F' ||
      where[0]=='f') {
    *b=0;
  } else {
    *b=1;
  }
  return 0;
}

int shell_write_boolean(igraph_bool_t b, const char *where) {
  int bb= b==0 ? 0 : 1;
  FILE *fout=shell_open_file(where, "w");
  int ret=fprintf(fout, "%i\n", bb);
  if (ret <= 0) { 
    fprintf(stderr, "Cannot write integer to '%s'\n", where);
    fclose(fout);
    exit(1);
  }  
  return 0;
}

int shell_read_real(igraph_real_t *b, const char *where) {
  int ret=sscanf(where, "%lf", b);
  if (ret == EOF || ret == 0) {
    fprintf(stderr, "Error, cannot interpret '%s' as real\n", where);
    exit(1);
  }  
  return 0;
}

int shell_write_real(igraph_real_t b, const char *where) {
  FILE *fout=shell_open_file(where, "w");
  int ret=fprintf(fout, "%g\n", (double)b);
  if (ret <= 0) { 
    fprintf(stderr, "Cannot write real to `%s'\n", where);
    fclose(fout);
    exit(1);
  }
  return 0;
}

int shell_read_enum(void* value, const char *where, ...) {
  int result=-1, *p=value;
  va_list args;
  va_start(args, where);
  
  while (1) {
    char *name=va_arg(args, char*);
    int code;
    if (name) {
      code=va_arg(args, int);
      if (!strcmp(optarg, name)) {
	result=code;
	break;
      }
    } else {
      break;
    }
  }
  
  if (result==-1) {
    fprintf(stderr, "Cannot interpret argument: '%s'.\n", where);
    exit(1);
  }   

  *p=result;

  return 0;
}

int shell_read_int(int *value, const char *where) {
  long int li=strtol(where, 0, 10);
  if (errno || li<INT_MIN || li>INT_MAX) {
    fprintf(stderr, "Integer too small/big: '%s'.\n", where);
  }
  *value=li;
  return 0;
}

int shell_read_longint(long int *value, const char *where) {
  long int li=strtol(where, 0, 10);
  if (errno) {
    fprintf(stderr, "Long integer too small/big: '%s'.\n", where);
  }
  *value=li;
  return 0;
}

int shell_read_file(FILE **file, const char *where, const char *mode) {
  *file=fopen(where, mode);
  if (!*file) {
    fprintf(stderr, "Cannot open file '%s'\n", where);
  }
  return 0;
}

int shell_read_matrixlist(igraph_vector_ptr_t *list, const char *where) {
  FILE *fin=shell_open_file(where, "r");
  igraph_vector_ptr_init(list, 0);
  shell_skip_whitespace(fin);
  while (!feof(fin)) {
    igraph_matrix_t *m=malloc(sizeof(igraph_matrix_t));
    shell_read_a_matrix(m, fin, where);
    igraph_vector_ptr_push_back(list, m);
    shell_skip_whitespace(fin);
  }
  fclose(fin);
  return 0;
}

int shell_read_graphlist(igraph_vector_ptr_t *list, const char *where) {
  FILE *fin=shell_open_file(where, "r");
  igraph_vector_ptr_init(list, 0);
  shell_skip_whitespace(fin);
  while (!feof(fin)) {
    igraph_t *g=malloc(sizeof(igraph_t));
    igraph_read_graph_graphml(g, fin, 0);
    igraph_vector_ptr_push_back(list, g);
    shell_skip_whitespace(fin);
  }
  fclose(fin);
  return 0;
}

int shell_read_strvector(igraph_strvector_t *str, const char *where) {
  FILE *fin=shell_open_file(where, "r");
  char *buffer=calloc(1000, sizeof(char));
  size_t size;
  int c;
  igraph_strvector_init(str, 0);
  while(1) {
    getline(&buffer, &size, fin);
    igraph_strvector_add(str, buffer);
    c=getc(fin);
    if (!feof(fin)) {
      ungetc(c, fin);
    } else {
      break;
    }
  }
  fclose(fin);
  free(buffer);
  
  return 0;
}

int shell_write_vectorlist(igraph_vector_ptr_t *v, const char *where) {
  FILE *fout=shell_open_file(where, "w");
  long int i, n=igraph_vector_ptr_size(v);
  for (i=0; i<n; i++) {
    shell_write_a_vector(VECTOR(*v)[i], fout, where);
  }
  fclose(fout);
  return 0;
}

int shell_write_vector_bool(igraph_vector_bool_t *v, const char *where) {
  FILE *fout=shell_open_file(where, "w");
  int ret=1;
  long int i, n=igraph_vector_bool_size(v);
  if (n>0) { ret=fprintf(fout, "%i", VECTOR(*v)[0]==0 ? 0 : 1); }
  if (ret <= 0) { 
    fprintf(stderr, "Cannot write vector to `%s'\n", where);
    fclose(fout);
    exit(1);
  }
  for (i=1; i<n; i++) {
    ret=fprintf(fout, " %i", VECTOR(*v)[i]==0 ? 0 : 1);
    if (ret <= 0) { 
      fprintf(stderr, "Cannot write vector to `%s'\n", where);
      fclose(fout);
      exit(1);
    }
  }
  fprintf(fout, "\n");
  fclose(fout);
  return 0;
}  

int shell_write_graphlist(igraph_vector_ptr_t *v, const char *where) {
  FILE *fout=shell_open_file(where, "w");
  long int i, n=igraph_vector_ptr_size(v);
  for (i=0; i<n; i++) {
    igraph_write_graph_graphml(VECTOR(*v)[i], fout);
  }
  fclose(fout);
  return 0;
}

int shell_free_graphlist(igraph_vector_ptr_t *v) {
  long int i, n=igraph_vector_ptr_size(v);
  for (i=0; i<n; i++) {
    igraph_destroy(VECTOR(*v)[i]);
    free(VECTOR(*v)[i]);
    VECTOR(*v)[i]=0;
  }
  return 0;
}

int shell_free_matrixlist(igraph_vector_ptr_t *v) {
  long int i, n=igraph_vector_ptr_size(v);
  for (i=0; i<n; i++) {
    igraph_matrix_destroy(VECTOR(*v)[i]);
    free(VECTOR(*v)[i]);
    VECTOR(*v)[i]=0;
  }
  return 0;
}

int shell_free_vectorlist(igraph_vector_ptr_t *v) {
  long int i, n=igraph_vector_ptr_size(v);
  for (i=0; i<n; i++) {
    igraph_vector_destroy(VECTOR(*v)[i]);
    free(VECTOR(*v)[i]);
    VECTOR(*v)[i]=0;
  }
  return 0;
}

/* ------------------------------------------------------------------------ */
/* Stimulus generated interface next                                        */
/* ------------------------------------------------------------------------ */


/* Function prototypes first */

int shell_igraph_empty(int argc, char **argv);
int shell_igraph_add_edges(int argc, char **argv);
int shell_igraph_add_vertices(int argc, char **argv);
int shell_igraph_delete_edges(int argc, char **argv);
int shell_igraph_delete_vertices(int argc, char **argv);
int shell_igraph_vcount(int argc, char **argv);
int shell_igraph_ecount(int argc, char **argv);
int shell_igraph_neighbors(int argc, char **argv);
int shell_igraph_is_directed(int argc, char **argv);
int shell_igraph_degree(int argc, char **argv);
int shell_igraph_edge(int argc, char **argv);
int shell_igraph_edges(int argc, char **argv);
int shell_igraph_get_eid(int argc, char **argv);
int shell_igraph_get_eids(int argc, char **argv);
int shell_igraph_adjacent(int argc, char **argv);
int shell_igraph_create(int argc, char **argv);
int shell_igraph_adjacency(int argc, char **argv);
int shell_igraph_star(int argc, char **argv);
int shell_igraph_lattice(int argc, char **argv);
int shell_igraph_ring(int argc, char **argv);
int shell_igraph_tree(int argc, char **argv);
int shell_igraph_full(int argc, char **argv);
int shell_igraph_full_citation(int argc, char **argv);
int shell_igraph_atlas(int argc, char **argv);
int shell_igraph_extended_chordal_ring(int argc, char **argv);
int shell_igraph_connect_neighborhood(int argc, char **argv);
int shell_igraph_linegraph(int argc, char **argv);
int shell_igraph_de_bruijn(int argc, char **argv);
int shell_igraph_kautz(int argc, char **argv);
int shell_igraph_famous(int argc, char **argv);
int shell_igraph_lcf_vector(int argc, char **argv);
int shell_igraph_adjlist(int argc, char **argv);
int shell_igraph_full_bipartite(int argc, char **argv);
int shell_igraph_barabasi_game(int argc, char **argv);
int shell_igraph_nonlinear_barabasi_game(int argc, char **argv);
int shell_igraph_erdos_renyi_game_gnp(int argc, char **argv);
int shell_igraph_erdos_renyi_game_gnm(int argc, char **argv);
int shell_igraph_degree_sequence_game(int argc, char **argv);
int shell_igraph_growing_random_game(int argc, char **argv);
int shell_igraph_barabasi_aging_game(int argc, char **argv);
int shell_igraph_recent_degree_game(int argc, char **argv);
int shell_igraph_recent_degree_aging_game(int argc, char **argv);
int shell_igraph_callaway_traits_game(int argc, char **argv);
int shell_igraph_establishment_game(int argc, char **argv);
int shell_igraph_grg_game(int argc, char **argv);
int shell_igraph_preference_game(int argc, char **argv);
int shell_igraph_asymmetric_preference_game(int argc, char **argv);
int shell_igraph_rewire_edges(int argc, char **argv);
int shell_igraph_watts_strogatz_game(int argc, char **argv);
int shell_igraph_lastcit_game(int argc, char **argv);
int shell_igraph_cited_type_game(int argc, char **argv);
int shell_igraph_citing_cited_type_game(int argc, char **argv);
int shell_igraph_forest_fire_game(int argc, char **argv);
int shell_igraph_are_connected(int argc, char **argv);
int shell_igraph_diameter(int argc, char **argv);
int shell_igraph_diameter_dijkstra(int argc, char **argv);
int shell_igraph_minimum_spanning_tree_unweighted(int argc, char **argv);
int shell_igraph_minimum_spanning_tree_prim(int argc, char **argv);
int shell_igraph_closeness(int argc, char **argv);
int shell_igraph_closeness_estimate(int argc, char **argv);
int shell_igraph_shortest_paths(int argc, char **argv);
int shell_igraph_get_shortest_paths(int argc, char **argv);
int shell_igraph_get_all_shortest_paths(int argc, char **argv);
int shell_igraph_shortest_paths_dijkstra(int argc, char **argv);
int shell_igraph_shortest_paths_bellman_ford(int argc, char **argv);
int shell_igraph_shortest_paths_johnson(int argc, char **argv);
int shell_igraph_subcomponent(int argc, char **argv);
int shell_igraph_betweenness(int argc, char **argv);
int shell_igraph_betweenness_estimate(int argc, char **argv);
int shell_igraph_edge_betweenness(int argc, char **argv);
int shell_igraph_edge_betweenness_estimate(int argc, char **argv);
int shell_igraph_pagerank_old(int argc, char **argv);
int shell_igraph_pagerank(int argc, char **argv);
int shell_igraph_rewire(int argc, char **argv);
int shell_igraph_subgraph(int argc, char **argv);
int shell_igraph_average_path_length(int argc, char **argv);
int shell_igraph_path_length_hist(int argc, char **argv);
int shell_igraph_simplify(int argc, char **argv);
int shell_igraph_transitivity_undirected(int argc, char **argv);
int shell_igraph_transitivity_local_undirected(int argc, char **argv);
int shell_igraph_transitivity_avglocal_undirected(int argc, char **argv);
int shell_igraph_transitivity_barrat(int argc, char **argv);
int shell_igraph_reciprocity(int argc, char **argv);
int shell_igraph_constraint(int argc, char **argv);
int shell_igraph_maxdegree(int argc, char **argv);
int shell_igraph_density(int argc, char **argv);
int shell_igraph_neighborhood_size(int argc, char **argv);
int shell_igraph_neighborhood(int argc, char **argv);
int shell_igraph_neighborhood_graphs(int argc, char **argv);
int shell_igraph_topological_sorting(int argc, char **argv);
int shell_igraph_is_loop(int argc, char **argv);
int shell_igraph_is_simple(int argc, char **argv);
int shell_igraph_is_multiple(int argc, char **argv);
int shell_igraph_count_multiple(int argc, char **argv);
int shell_igraph_girth(int argc, char **argv);
int shell_igraph_add_edge(int argc, char **argv);
int shell_igraph_eigenvector_centrality(int argc, char **argv);
int shell_igraph_hub_score(int argc, char **argv);
int shell_igraph_authority_score(int argc, char **argv);
int shell_igraph_arpack_rssolve(int argc, char **argv);
int shell_igraph_arpack_rnsolve(int argc, char **argv);
int shell_igraph_arpack_unpack_complex(int argc, char **argv);
int shell_igraph_unfold_tree(int argc, char **argv);
int shell_igraph_is_mutual(int argc, char **argv);
int shell_igraph_maximum_cardinality_search(int argc, char **argv);
int shell_igraph_is_chordal(int argc, char **argv);
int shell_igraph_avg_nearest_neighbor_degree(int argc, char **argv);
int shell_igraph_strength(int argc, char **argv);
int shell_igraph_centralization(int argc, char **argv);
int shell_igraph_centralization_degree(int argc, char **argv);
int shell_igraph_centralization_betweenness(int argc, char **argv);
int shell_igraph_centralization_closeness(int argc, char **argv);
int shell_igraph_centralization_eigenvector_centrality(int argc, char **argv);
int shell_igraph_bfs(int argc, char **argv);
int shell_igraph_dfs(int argc, char **argv);
int shell_igraph_bipartite_projection_size(int argc, char **argv);
int shell_igraph_bipartite_projection(int argc, char **argv);
int shell_igraph_create_bipartite(int argc, char **argv);
int shell_igraph_incidence(int argc, char **argv);
int shell_igraph_get_incidence(int argc, char **argv);
int shell_igraph_is_bipartite(int argc, char **argv);
int shell_igraph_laplacian(int argc, char **argv);
int shell_igraph_clusters(int argc, char **argv);
int shell_igraph_is_connected(int argc, char **argv);
int shell_igraph_decompose(int argc, char **argv);
int shell_igraph_articulation_points(int argc, char **argv);
int shell_igraph_biconnected_components(int argc, char **argv);
int shell_igraph_cliques(int argc, char **argv);
int shell_igraph_largest_cliques(int argc, char **argv);
int shell_igraph_maximal_cliques(int argc, char **argv);
int shell_igraph_clique_number(int argc, char **argv);
int shell_igraph_independent_vertex_sets(int argc, char **argv);
int shell_igraph_largest_independent_vertex_sets(int argc, char **argv);
int shell_igraph_maximal_independent_vertex_sets(int argc, char **argv);
int shell_igraph_independence_number(int argc, char **argv);
int shell_igraph_layout_random(int argc, char **argv);
int shell_igraph_layout_circle(int argc, char **argv);
int shell_igraph_layout_fruchterman_reingold(int argc, char **argv);
int shell_igraph_layout_grid_fruchterman_reingold(int argc, char **argv);
int shell_igraph_layout_kamada_kawai(int argc, char **argv);
int shell_igraph_layout_lgl(int argc, char **argv);
int shell_igraph_layout_reingold_tilford(int argc, char **argv);
int shell_igraph_layout_reingold_tilford_circular(int argc, char **argv);
int shell_igraph_layout_random_3d(int argc, char **argv);
int shell_igraph_layout_sphere(int argc, char **argv);
int shell_igraph_layout_fruchterman_reingold_3d(int argc, char **argv);
int shell_igraph_layout_kamada_kawai_3d(int argc, char **argv);
int shell_igraph_layout_graphopt(int argc, char **argv);
int shell_igraph_layout_drl(int argc, char **argv);
int shell_igraph_layout_drl_3d(int argc, char **argv);
int shell_igraph_layout_merge_dla(int argc, char **argv);
int shell_igraph_cocitation(int argc, char **argv);
int shell_igraph_bibcoupling(int argc, char **argv);
int shell_igraph_similarity_jaccard(int argc, char **argv);
int shell_igraph_similarity_dice(int argc, char **argv);
int shell_igraph_similarity_inverse_log_weighted(int argc, char **argv);
int shell_igraph_community_spinglass(int argc, char **argv);
int shell_igraph_community_spinglass_single(int argc, char **argv);
int shell_igraph_community_walktrap(int argc, char **argv);
int shell_igraph_community_edge_betweenness(int argc, char **argv);
int shell_igraph_community_eb_get_merges(int argc, char **argv);
int shell_igraph_community_fastgreedy(int argc, char **argv);
int shell_igraph_community_to_membership(int argc, char **argv);
int shell_igraph_le_community_to_membership(int argc, char **argv);
int shell_igraph_modularity(int argc, char **argv);
int shell_igraph_community_leading_eigenvector(int argc, char **argv);
int shell_igraph_community_leading_eigenvector_naive(int argc, char **argv);
int shell_igraph_community_leading_eigenvector_step(int argc, char **argv);
int shell_igraph_community_label_propagation(int argc, char **argv);
int shell_igraph_get_adjacency(int argc, char **argv);
int shell_igraph_get_edgelist(int argc, char **argv);
int shell_igraph_to_directed(int argc, char **argv);
int shell_igraph_to_undirected(int argc, char **argv);
int shell_igraph_read_graph_edgelist(int argc, char **argv);
int shell_igraph_read_graph_ncol(int argc, char **argv);
int shell_igraph_read_graph_lgl(int argc, char **argv);
int shell_igraph_read_graph_pajek(int argc, char **argv);
int shell_igraph_read_graph_graphml(int argc, char **argv);
int shell_igraph_read_graph_dimacs(int argc, char **argv);
int shell_igraph_read_graph_graphdb(int argc, char **argv);
int shell_igraph_read_graph_gml(int argc, char **argv);
int shell_igraph_read_graph_dl(int argc, char **argv);
int shell_igraph_write_graph_edgelist(int argc, char **argv);
int shell_igraph_write_graph_ncol(int argc, char **argv);
int shell_igraph_write_graph_lgl(int argc, char **argv);
int shell_igraph_write_graph_graphml(int argc, char **argv);
int shell_igraph_write_graph_pajek(int argc, char **argv);
int shell_igraph_write_graph_dimacs(int argc, char **argv);
int shell_igraph_write_graph_gml(int argc, char **argv);
int shell_igraph_write_graph_dot(int argc, char **argv);
int shell_igraph_motifs_randesu(int argc, char **argv);
int shell_igraph_motifs_randesu_estimate(int argc, char **argv);
int shell_igraph_motifs_randesu_no(int argc, char **argv);
int shell_igraph_dyad_census(int argc, char **argv);
int shell_igraph_triad_census(int argc, char **argv);
int shell_igraph_disjoint_union(int argc, char **argv);
int shell_igraph_disjoint_union_many(int argc, char **argv);
int shell_igraph_union(int argc, char **argv);
int shell_igraph_union_many(int argc, char **argv);
int shell_igraph_intersection(int argc, char **argv);
int shell_igraph_intersection_many(int argc, char **argv);
int shell_igraph_difference(int argc, char **argv);
int shell_igraph_complementer(int argc, char **argv);
int shell_igraph_compose(int argc, char **argv);
int shell_igraph_maxflow_value(int argc, char **argv);
int shell_igraph_mincut_value(int argc, char **argv);
int shell_igraph_st_mincut_value(int argc, char **argv);
int shell_igraph_mincut(int argc, char **argv);
int shell_igraph_st_vertex_connectivity(int argc, char **argv);
int shell_igraph_vertex_connectivity(int argc, char **argv);
int shell_igraph_st_edge_connectivity(int argc, char **argv);
int shell_igraph_edge_connectivity(int argc, char **argv);
int shell_igraph_edge_disjoint_paths(int argc, char **argv);
int shell_igraph_vertex_disjoint_paths(int argc, char **argv);
int shell_igraph_adhesion(int argc, char **argv);
int shell_igraph_cohesion(int argc, char **argv);
int shell_igraph_coreness(int argc, char **argv);
int shell_igraph_isoclass(int argc, char **argv);
int shell_igraph_isomorphic(int argc, char **argv);
int shell_igraph_isoclass_subgraph(int argc, char **argv);
int shell_igraph_isoclass_create(int argc, char **argv);
int shell_igraph_isomorphic_vf2(int argc, char **argv);
int shell_igraph_count_isomorphisms_vf2(int argc, char **argv);
int shell_igraph_get_isomorphisms_vf2(int argc, char **argv);
int shell_igraph_subisomorphic_vf2(int argc, char **argv);
int shell_igraph_count_subisomorphisms_vf2(int argc, char **argv);
int shell_igraph_get_subisomorphisms_vf2(int argc, char **argv);
int shell_igraph_isomorphic_34(int argc, char **argv);
int shell_igraph_canonical_permutation(int argc, char **argv);
int shell_igraph_permute_vertices(int argc, char **argv);
int shell_igraph_isomorphic_bliss(int argc, char **argv);
int shell_igraph_automorphisms(int argc, char **argv);
int shell_igraph_running_mean(int argc, char **argv);
int shell_igraph_random_sample(int argc, char **argv);
int shell_igraph_convex_hull(int argc, char **argv);
int shell_igraph_revolver_ml_d(int argc, char **argv);
int shell_igraph_revolver_probs_d(int argc, char **argv);
int shell_igraph_revolver_ml_de(int argc, char **argv);
int shell_igraph_revolver_probs_de(int argc, char **argv);
int shell_igraph_revolver_ml_ade(int argc, char **argv);
int shell_igraph_revolver_probs_ade(int argc, char **argv);
int shell_igraph_revolver_ml_f(int argc, char **argv);
int shell_igraph_revolver_ml_df(int argc, char **argv);
int shell_igraph_revolver_ml_l(int argc, char **argv);
int shell_igraph_revolver_ml_ad(int argc, char **argv);
int shell_igraph_revolver_probs_ad(int argc, char **argv);
int shell_igraph_revolver_ml_D_alpha(int argc, char **argv);
int shell_igraph_revolver_ml_D_alpha_a(int argc, char **argv);
int shell_igraph_revolver_ml_DE_alpha_a(int argc, char **argv);
int shell_igraph_revolver_ml_AD_alpha_a_beta(int argc, char **argv);
int shell_igraph_revolver_ml_AD_dpareto(int argc, char **argv);
int shell_igraph_revolver_ml_AD_dpareto_eval(int argc, char **argv);
int shell_igraph_revolver_ml_ADE_alpha_a_beta(int argc, char **argv);
int shell_igraph_revolver_ml_ADE_dpareto(int argc, char **argv);
int shell_igraph_revolver_ml_ADE_dpareto_eval(int argc, char **argv);
int shell_igraph_revolver_ml_ADE_dpareto_evalf(int argc, char **argv);
int shell_igraph_revolver_probs_ADE_dpareto(int argc, char **argv);
int shell_igraph_convergence_degree(int argc, char **argv);

/* The main function */

int main(int argc, char **argv) {

  const char *base=basename(argv[0]);

  if (!strcasecmp(base, "igraph_empty")) {
    return shell_igraph_empty(argc, argv);
  } else if (!strcasecmp(base, "igraph_add_edges")) {
    return shell_igraph_add_edges(argc, argv);
  } else if (!strcasecmp(base, "igraph_add_vertices")) {
    return shell_igraph_add_vertices(argc, argv);
  } else if (!strcasecmp(base, "igraph_delete_edges")) {
    return shell_igraph_delete_edges(argc, argv);
  } else if (!strcasecmp(base, "igraph_delete_vertices")) {
    return shell_igraph_delete_vertices(argc, argv);
  } else if (!strcasecmp(base, "igraph_vcount")) {
    return shell_igraph_vcount(argc, argv);
  } else if (!strcasecmp(base, "igraph_ecount")) {
    return shell_igraph_ecount(argc, argv);
  } else if (!strcasecmp(base, "igraph_neighbors")) {
    return shell_igraph_neighbors(argc, argv);
  } else if (!strcasecmp(base, "igraph_is_directed")) {
    return shell_igraph_is_directed(argc, argv);
  } else if (!strcasecmp(base, "igraph_degree")) {
    return shell_igraph_degree(argc, argv);
  } else if (!strcasecmp(base, "igraph_edge")) {
    return shell_igraph_edge(argc, argv);
  } else if (!strcasecmp(base, "igraph_edges")) {
    return shell_igraph_edges(argc, argv);
  } else if (!strcasecmp(base, "igraph_get_eid")) {
    return shell_igraph_get_eid(argc, argv);
  } else if (!strcasecmp(base, "igraph_get_eids")) {
    return shell_igraph_get_eids(argc, argv);
  } else if (!strcasecmp(base, "igraph_adjacent")) {
    return shell_igraph_adjacent(argc, argv);
  } else if (!strcasecmp(base, "igraph_create")) {
    return shell_igraph_create(argc, argv);
  } else if (!strcasecmp(base, "igraph_adjacency")) {
    return shell_igraph_adjacency(argc, argv);
  } else if (!strcasecmp(base, "igraph_star")) {
    return shell_igraph_star(argc, argv);
  } else if (!strcasecmp(base, "igraph_lattice")) {
    return shell_igraph_lattice(argc, argv);
  } else if (!strcasecmp(base, "igraph_ring")) {
    return shell_igraph_ring(argc, argv);
  } else if (!strcasecmp(base, "igraph_tree")) {
    return shell_igraph_tree(argc, argv);
  } else if (!strcasecmp(base, "igraph_full")) {
    return shell_igraph_full(argc, argv);
  } else if (!strcasecmp(base, "igraph_full_citation")) {
    return shell_igraph_full_citation(argc, argv);
  } else if (!strcasecmp(base, "igraph_atlas")) {
    return shell_igraph_atlas(argc, argv);
  } else if (!strcasecmp(base, "igraph_extended_chordal_ring")) {
    return shell_igraph_extended_chordal_ring(argc, argv);
  } else if (!strcasecmp(base, "igraph_connect_neighborhood")) {
    return shell_igraph_connect_neighborhood(argc, argv);
  } else if (!strcasecmp(base, "igraph_linegraph")) {
    return shell_igraph_linegraph(argc, argv);
  } else if (!strcasecmp(base, "igraph_de_bruijn")) {
    return shell_igraph_de_bruijn(argc, argv);
  } else if (!strcasecmp(base, "igraph_kautz")) {
    return shell_igraph_kautz(argc, argv);
  } else if (!strcasecmp(base, "igraph_famous")) {
    return shell_igraph_famous(argc, argv);
  } else if (!strcasecmp(base, "igraph_lcf_vector")) {
    return shell_igraph_lcf_vector(argc, argv);
  } else if (!strcasecmp(base, "igraph_adjlist")) {
    return shell_igraph_adjlist(argc, argv);
  } else if (!strcasecmp(base, "igraph_full_bipartite")) {
    return shell_igraph_full_bipartite(argc, argv);
  } else if (!strcasecmp(base, "igraph_barabasi_game")) {
    return shell_igraph_barabasi_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_nonlinear_barabasi_game")) {
    return shell_igraph_nonlinear_barabasi_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_erdos_renyi_game_gnp")) {
    return shell_igraph_erdos_renyi_game_gnp(argc, argv);
  } else if (!strcasecmp(base, "igraph_erdos_renyi_game_gnm")) {
    return shell_igraph_erdos_renyi_game_gnm(argc, argv);
  } else if (!strcasecmp(base, "igraph_degree_sequence_game")) {
    return shell_igraph_degree_sequence_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_growing_random_game")) {
    return shell_igraph_growing_random_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_barabasi_aging_game")) {
    return shell_igraph_barabasi_aging_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_recent_degree_game")) {
    return shell_igraph_recent_degree_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_recent_degree_aging_game")) {
    return shell_igraph_recent_degree_aging_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_callaway_traits_game")) {
    return shell_igraph_callaway_traits_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_establishment_game")) {
    return shell_igraph_establishment_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_grg_game")) {
    return shell_igraph_grg_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_preference_game")) {
    return shell_igraph_preference_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_asymmetric_preference_game")) {
    return shell_igraph_asymmetric_preference_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_rewire_edges")) {
    return shell_igraph_rewire_edges(argc, argv);
  } else if (!strcasecmp(base, "igraph_watts_strogatz_game")) {
    return shell_igraph_watts_strogatz_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_lastcit_game")) {
    return shell_igraph_lastcit_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_cited_type_game")) {
    return shell_igraph_cited_type_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_citing_cited_type_game")) {
    return shell_igraph_citing_cited_type_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_forest_fire_game")) {
    return shell_igraph_forest_fire_game(argc, argv);
  } else if (!strcasecmp(base, "igraph_are_connected")) {
    return shell_igraph_are_connected(argc, argv);
  } else if (!strcasecmp(base, "igraph_diameter")) {
    return shell_igraph_diameter(argc, argv);
  } else if (!strcasecmp(base, "igraph_diameter_dijkstra")) {
    return shell_igraph_diameter_dijkstra(argc, argv);
  } else if (!strcasecmp(base, "igraph_minimum_spanning_tree_unweighted")) {
    return shell_igraph_minimum_spanning_tree_unweighted(argc, argv);
  } else if (!strcasecmp(base, "igraph_minimum_spanning_tree_prim")) {
    return shell_igraph_minimum_spanning_tree_prim(argc, argv);
  } else if (!strcasecmp(base, "igraph_closeness")) {
    return shell_igraph_closeness(argc, argv);
  } else if (!strcasecmp(base, "igraph_closeness_estimate")) {
    return shell_igraph_closeness_estimate(argc, argv);
  } else if (!strcasecmp(base, "igraph_shortest_paths")) {
    return shell_igraph_shortest_paths(argc, argv);
  } else if (!strcasecmp(base, "igraph_get_shortest_paths")) {
    return shell_igraph_get_shortest_paths(argc, argv);
  } else if (!strcasecmp(base, "igraph_get_all_shortest_paths")) {
    return shell_igraph_get_all_shortest_paths(argc, argv);
  } else if (!strcasecmp(base, "igraph_shortest_paths_dijkstra")) {
    return shell_igraph_shortest_paths_dijkstra(argc, argv);
  } else if (!strcasecmp(base, "igraph_shortest_paths_bellman_ford")) {
    return shell_igraph_shortest_paths_bellman_ford(argc, argv);
  } else if (!strcasecmp(base, "igraph_shortest_paths_johnson")) {
    return shell_igraph_shortest_paths_johnson(argc, argv);
  } else if (!strcasecmp(base, "igraph_subcomponent")) {
    return shell_igraph_subcomponent(argc, argv);
  } else if (!strcasecmp(base, "igraph_betweenness")) {
    return shell_igraph_betweenness(argc, argv);
  } else if (!strcasecmp(base, "igraph_betweenness_estimate")) {
    return shell_igraph_betweenness_estimate(argc, argv);
  } else if (!strcasecmp(base, "igraph_edge_betweenness")) {
    return shell_igraph_edge_betweenness(argc, argv);
  } else if (!strcasecmp(base, "igraph_edge_betweenness_estimate")) {
    return shell_igraph_edge_betweenness_estimate(argc, argv);
  } else if (!strcasecmp(base, "igraph_pagerank_old")) {
    return shell_igraph_pagerank_old(argc, argv);
  } else if (!strcasecmp(base, "igraph_pagerank")) {
    return shell_igraph_pagerank(argc, argv);
  } else if (!strcasecmp(base, "igraph_rewire")) {
    return shell_igraph_rewire(argc, argv);
  } else if (!strcasecmp(base, "igraph_subgraph")) {
    return shell_igraph_subgraph(argc, argv);
  } else if (!strcasecmp(base, "igraph_average_path_length")) {
    return shell_igraph_average_path_length(argc, argv);
  } else if (!strcasecmp(base, "igraph_path_length_hist")) {
    return shell_igraph_path_length_hist(argc, argv);
  } else if (!strcasecmp(base, "igraph_simplify")) {
    return shell_igraph_simplify(argc, argv);
  } else if (!strcasecmp(base, "igraph_transitivity_undirected")) {
    return shell_igraph_transitivity_undirected(argc, argv);
  } else if (!strcasecmp(base, "igraph_transitivity_local_undirected")) {
    return shell_igraph_transitivity_local_undirected(argc, argv);
  } else if (!strcasecmp(base, "igraph_transitivity_avglocal_undirected")) {
    return shell_igraph_transitivity_avglocal_undirected(argc, argv);
  } else if (!strcasecmp(base, "igraph_transitivity_barrat")) {
    return shell_igraph_transitivity_barrat(argc, argv);
  } else if (!strcasecmp(base, "igraph_reciprocity")) {
    return shell_igraph_reciprocity(argc, argv);
  } else if (!strcasecmp(base, "igraph_constraint")) {
    return shell_igraph_constraint(argc, argv);
  } else if (!strcasecmp(base, "igraph_maxdegree")) {
    return shell_igraph_maxdegree(argc, argv);
  } else if (!strcasecmp(base, "igraph_density")) {
    return shell_igraph_density(argc, argv);
  } else if (!strcasecmp(base, "igraph_neighborhood_size")) {
    return shell_igraph_neighborhood_size(argc, argv);
  } else if (!strcasecmp(base, "igraph_neighborhood")) {
    return shell_igraph_neighborhood(argc, argv);
  } else if (!strcasecmp(base, "igraph_neighborhood_graphs")) {
    return shell_igraph_neighborhood_graphs(argc, argv);
  } else if (!strcasecmp(base, "igraph_topological_sorting")) {
    return shell_igraph_topological_sorting(argc, argv);
  } else if (!strcasecmp(base, "igraph_is_loop")) {
    return shell_igraph_is_loop(argc, argv);
  } else if (!strcasecmp(base, "igraph_is_simple")) {
    return shell_igraph_is_simple(argc, argv);
  } else if (!strcasecmp(base, "igraph_is_multiple")) {
    return shell_igraph_is_multiple(argc, argv);
  } else if (!strcasecmp(base, "igraph_count_multiple")) {
    return shell_igraph_count_multiple(argc, argv);
  } else if (!strcasecmp(base, "igraph_girth")) {
    return shell_igraph_girth(argc, argv);
  } else if (!strcasecmp(base, "igraph_add_edge")) {
    return shell_igraph_add_edge(argc, argv);
  } else if (!strcasecmp(base, "igraph_eigenvector_centrality")) {
    return shell_igraph_eigenvector_centrality(argc, argv);
  } else if (!strcasecmp(base, "igraph_hub_score")) {
    return shell_igraph_hub_score(argc, argv);
  } else if (!strcasecmp(base, "igraph_authority_score")) {
    return shell_igraph_authority_score(argc, argv);
  } else if (!strcasecmp(base, "igraph_arpack_rssolve")) {
    return shell_igraph_arpack_rssolve(argc, argv);
  } else if (!strcasecmp(base, "igraph_arpack_rnsolve")) {
    return shell_igraph_arpack_rnsolve(argc, argv);
  } else if (!strcasecmp(base, "igraph_arpack_unpack_complex")) {
    return shell_igraph_arpack_unpack_complex(argc, argv);
  } else if (!strcasecmp(base, "igraph_unfold_tree")) {
    return shell_igraph_unfold_tree(argc, argv);
  } else if (!strcasecmp(base, "igraph_is_mutual")) {
    return shell_igraph_is_mutual(argc, argv);
  } else if (!strcasecmp(base, "igraph_maximum_cardinality_search")) {
    return shell_igraph_maximum_cardinality_search(argc, argv);
  } else if (!strcasecmp(base, "igraph_is_chordal")) {
    return shell_igraph_is_chordal(argc, argv);
  } else if (!strcasecmp(base, "igraph_avg_nearest_neighbor_degree")) {
    return shell_igraph_avg_nearest_neighbor_degree(argc, argv);
  } else if (!strcasecmp(base, "igraph_strength")) {
    return shell_igraph_strength(argc, argv);
  } else if (!strcasecmp(base, "igraph_centralization")) {
    return shell_igraph_centralization(argc, argv);
  } else if (!strcasecmp(base, "igraph_centralization_degree")) {
    return shell_igraph_centralization_degree(argc, argv);
  } else if (!strcasecmp(base, "igraph_centralization_betweenness")) {
    return shell_igraph_centralization_betweenness(argc, argv);
  } else if (!strcasecmp(base, "igraph_centralization_closeness")) {
    return shell_igraph_centralization_closeness(argc, argv);
  } else if (!strcasecmp(base, "igraph_centralization_eigenvector_centrality")) {
    return shell_igraph_centralization_eigenvector_centrality(argc, argv);
  } else if (!strcasecmp(base, "igraph_bfs")) {
    return shell_igraph_bfs(argc, argv);
  } else if (!strcasecmp(base, "igraph_dfs")) {
    return shell_igraph_dfs(argc, argv);
  } else if (!strcasecmp(base, "igraph_bipartite_projection_size")) {
    return shell_igraph_bipartite_projection_size(argc, argv);
  } else if (!strcasecmp(base, "igraph_bipartite_projection")) {
    return shell_igraph_bipartite_projection(argc, argv);
  } else if (!strcasecmp(base, "igraph_create_bipartite")) {
    return shell_igraph_create_bipartite(argc, argv);
  } else if (!strcasecmp(base, "igraph_incidence")) {
    return shell_igraph_incidence(argc, argv);
  } else if (!strcasecmp(base, "igraph_get_incidence")) {
    return shell_igraph_get_incidence(argc, argv);
  } else if (!strcasecmp(base, "igraph_is_bipartite")) {
    return shell_igraph_is_bipartite(argc, argv);
  } else if (!strcasecmp(base, "igraph_laplacian")) {
    return shell_igraph_laplacian(argc, argv);
  } else if (!strcasecmp(base, "igraph_clusters")) {
    return shell_igraph_clusters(argc, argv);
  } else if (!strcasecmp(base, "igraph_is_connected")) {
    return shell_igraph_is_connected(argc, argv);
  } else if (!strcasecmp(base, "igraph_decompose")) {
    return shell_igraph_decompose(argc, argv);
  } else if (!strcasecmp(base, "igraph_articulation_points")) {
    return shell_igraph_articulation_points(argc, argv);
  } else if (!strcasecmp(base, "igraph_biconnected_components")) {
    return shell_igraph_biconnected_components(argc, argv);
  } else if (!strcasecmp(base, "igraph_cliques")) {
    return shell_igraph_cliques(argc, argv);
  } else if (!strcasecmp(base, "igraph_largest_cliques")) {
    return shell_igraph_largest_cliques(argc, argv);
  } else if (!strcasecmp(base, "igraph_maximal_cliques")) {
    return shell_igraph_maximal_cliques(argc, argv);
  } else if (!strcasecmp(base, "igraph_clique_number")) {
    return shell_igraph_clique_number(argc, argv);
  } else if (!strcasecmp(base, "igraph_independent_vertex_sets")) {
    return shell_igraph_independent_vertex_sets(argc, argv);
  } else if (!strcasecmp(base, "igraph_largest_independent_vertex_sets")) {
    return shell_igraph_largest_independent_vertex_sets(argc, argv);
  } else if (!strcasecmp(base, "igraph_maximal_independent_vertex_sets")) {
    return shell_igraph_maximal_independent_vertex_sets(argc, argv);
  } else if (!strcasecmp(base, "igraph_independence_number")) {
    return shell_igraph_independence_number(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_random")) {
    return shell_igraph_layout_random(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_circle")) {
    return shell_igraph_layout_circle(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_fruchterman_reingold")) {
    return shell_igraph_layout_fruchterman_reingold(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_grid_fruchterman_reingold")) {
    return shell_igraph_layout_grid_fruchterman_reingold(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_kamada_kawai")) {
    return shell_igraph_layout_kamada_kawai(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_lgl")) {
    return shell_igraph_layout_lgl(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_reingold_tilford")) {
    return shell_igraph_layout_reingold_tilford(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_reingold_tilford_circular")) {
    return shell_igraph_layout_reingold_tilford_circular(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_random_3d")) {
    return shell_igraph_layout_random_3d(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_sphere")) {
    return shell_igraph_layout_sphere(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_fruchterman_reingold_3d")) {
    return shell_igraph_layout_fruchterman_reingold_3d(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_kamada_kawai_3d")) {
    return shell_igraph_layout_kamada_kawai_3d(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_graphopt")) {
    return shell_igraph_layout_graphopt(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_drl")) {
    return shell_igraph_layout_drl(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_drl_3d")) {
    return shell_igraph_layout_drl_3d(argc, argv);
  } else if (!strcasecmp(base, "igraph_layout_merge_dla")) {
    return shell_igraph_layout_merge_dla(argc, argv);
  } else if (!strcasecmp(base, "igraph_cocitation")) {
    return shell_igraph_cocitation(argc, argv);
  } else if (!strcasecmp(base, "igraph_bibcoupling")) {
    return shell_igraph_bibcoupling(argc, argv);
  } else if (!strcasecmp(base, "igraph_similarity_jaccard")) {
    return shell_igraph_similarity_jaccard(argc, argv);
  } else if (!strcasecmp(base, "igraph_similarity_dice")) {
    return shell_igraph_similarity_dice(argc, argv);
  } else if (!strcasecmp(base, "igraph_similarity_inverse_log_weighted")) {
    return shell_igraph_similarity_inverse_log_weighted(argc, argv);
  } else if (!strcasecmp(base, "igraph_community_spinglass")) {
    return shell_igraph_community_spinglass(argc, argv);
  } else if (!strcasecmp(base, "igraph_community_spinglass_single")) {
    return shell_igraph_community_spinglass_single(argc, argv);
  } else if (!strcasecmp(base, "igraph_community_walktrap")) {
    return shell_igraph_community_walktrap(argc, argv);
  } else if (!strcasecmp(base, "igraph_community_edge_betweenness")) {
    return shell_igraph_community_edge_betweenness(argc, argv);
  } else if (!strcasecmp(base, "igraph_community_eb_get_merges")) {
    return shell_igraph_community_eb_get_merges(argc, argv);
  } else if (!strcasecmp(base, "igraph_community_fastgreedy")) {
    return shell_igraph_community_fastgreedy(argc, argv);
  } else if (!strcasecmp(base, "igraph_community_to_membership")) {
    return shell_igraph_community_to_membership(argc, argv);
  } else if (!strcasecmp(base, "igraph_le_community_to_membership")) {
    return shell_igraph_le_community_to_membership(argc, argv);
  } else if (!strcasecmp(base, "igraph_modularity")) {
    return shell_igraph_modularity(argc, argv);
  } else if (!strcasecmp(base, "igraph_community_leading_eigenvector")) {
    return shell_igraph_community_leading_eigenvector(argc, argv);
  } else if (!strcasecmp(base, "igraph_community_leading_eigenvector_naive")) {
    return shell_igraph_community_leading_eigenvector_naive(argc, argv);
  } else if (!strcasecmp(base, "igraph_community_leading_eigenvector_step")) {
    return shell_igraph_community_leading_eigenvector_step(argc, argv);
  } else if (!strcasecmp(base, "igraph_community_label_propagation")) {
    return shell_igraph_community_label_propagation(argc, argv);
  } else if (!strcasecmp(base, "igraph_get_adjacency")) {
    return shell_igraph_get_adjacency(argc, argv);
  } else if (!strcasecmp(base, "igraph_get_edgelist")) {
    return shell_igraph_get_edgelist(argc, argv);
  } else if (!strcasecmp(base, "igraph_to_directed")) {
    return shell_igraph_to_directed(argc, argv);
  } else if (!strcasecmp(base, "igraph_to_undirected")) {
    return shell_igraph_to_undirected(argc, argv);
  } else if (!strcasecmp(base, "igraph_read_graph_edgelist")) {
    return shell_igraph_read_graph_edgelist(argc, argv);
  } else if (!strcasecmp(base, "igraph_read_graph_ncol")) {
    return shell_igraph_read_graph_ncol(argc, argv);
  } else if (!strcasecmp(base, "igraph_read_graph_lgl")) {
    return shell_igraph_read_graph_lgl(argc, argv);
  } else if (!strcasecmp(base, "igraph_read_graph_pajek")) {
    return shell_igraph_read_graph_pajek(argc, argv);
  } else if (!strcasecmp(base, "igraph_read_graph_graphml")) {
    return shell_igraph_read_graph_graphml(argc, argv);
  } else if (!strcasecmp(base, "igraph_read_graph_dimacs")) {
    return shell_igraph_read_graph_dimacs(argc, argv);
  } else if (!strcasecmp(base, "igraph_read_graph_graphdb")) {
    return shell_igraph_read_graph_graphdb(argc, argv);
  } else if (!strcasecmp(base, "igraph_read_graph_gml")) {
    return shell_igraph_read_graph_gml(argc, argv);
  } else if (!strcasecmp(base, "igraph_read_graph_dl")) {
    return shell_igraph_read_graph_dl(argc, argv);
  } else if (!strcasecmp(base, "igraph_write_graph_edgelist")) {
    return shell_igraph_write_graph_edgelist(argc, argv);
  } else if (!strcasecmp(base, "igraph_write_graph_ncol")) {
    return shell_igraph_write_graph_ncol(argc, argv);
  } else if (!strcasecmp(base, "igraph_write_graph_lgl")) {
    return shell_igraph_write_graph_lgl(argc, argv);
  } else if (!strcasecmp(base, "igraph_write_graph_graphml")) {
    return shell_igraph_write_graph_graphml(argc, argv);
  } else if (!strcasecmp(base, "igraph_write_graph_pajek")) {
    return shell_igraph_write_graph_pajek(argc, argv);
  } else if (!strcasecmp(base, "igraph_write_graph_dimacs")) {
    return shell_igraph_write_graph_dimacs(argc, argv);
  } else if (!strcasecmp(base, "igraph_write_graph_gml")) {
    return shell_igraph_write_graph_gml(argc, argv);
  } else if (!strcasecmp(base, "igraph_write_graph_dot")) {
    return shell_igraph_write_graph_dot(argc, argv);
  } else if (!strcasecmp(base, "igraph_motifs_randesu")) {
    return shell_igraph_motifs_randesu(argc, argv);
  } else if (!strcasecmp(base, "igraph_motifs_randesu_estimate")) {
    return shell_igraph_motifs_randesu_estimate(argc, argv);
  } else if (!strcasecmp(base, "igraph_motifs_randesu_no")) {
    return shell_igraph_motifs_randesu_no(argc, argv);
  } else if (!strcasecmp(base, "igraph_dyad_census")) {
    return shell_igraph_dyad_census(argc, argv);
  } else if (!strcasecmp(base, "igraph_triad_census")) {
    return shell_igraph_triad_census(argc, argv);
  } else if (!strcasecmp(base, "igraph_disjoint_union")) {
    return shell_igraph_disjoint_union(argc, argv);
  } else if (!strcasecmp(base, "igraph_disjoint_union_many")) {
    return shell_igraph_disjoint_union_many(argc, argv);
  } else if (!strcasecmp(base, "igraph_union")) {
    return shell_igraph_union(argc, argv);
  } else if (!strcasecmp(base, "igraph_union_many")) {
    return shell_igraph_union_many(argc, argv);
  } else if (!strcasecmp(base, "igraph_intersection")) {
    return shell_igraph_intersection(argc, argv);
  } else if (!strcasecmp(base, "igraph_intersection_many")) {
    return shell_igraph_intersection_many(argc, argv);
  } else if (!strcasecmp(base, "igraph_difference")) {
    return shell_igraph_difference(argc, argv);
  } else if (!strcasecmp(base, "igraph_complementer")) {
    return shell_igraph_complementer(argc, argv);
  } else if (!strcasecmp(base, "igraph_compose")) {
    return shell_igraph_compose(argc, argv);
  } else if (!strcasecmp(base, "igraph_maxflow_value")) {
    return shell_igraph_maxflow_value(argc, argv);
  } else if (!strcasecmp(base, "igraph_mincut_value")) {
    return shell_igraph_mincut_value(argc, argv);
  } else if (!strcasecmp(base, "igraph_st_mincut_value")) {
    return shell_igraph_st_mincut_value(argc, argv);
  } else if (!strcasecmp(base, "igraph_mincut")) {
    return shell_igraph_mincut(argc, argv);
  } else if (!strcasecmp(base, "igraph_st_vertex_connectivity")) {
    return shell_igraph_st_vertex_connectivity(argc, argv);
  } else if (!strcasecmp(base, "igraph_vertex_connectivity")) {
    return shell_igraph_vertex_connectivity(argc, argv);
  } else if (!strcasecmp(base, "igraph_st_edge_connectivity")) {
    return shell_igraph_st_edge_connectivity(argc, argv);
  } else if (!strcasecmp(base, "igraph_edge_connectivity")) {
    return shell_igraph_edge_connectivity(argc, argv);
  } else if (!strcasecmp(base, "igraph_edge_disjoint_paths")) {
    return shell_igraph_edge_disjoint_paths(argc, argv);
  } else if (!strcasecmp(base, "igraph_vertex_disjoint_paths")) {
    return shell_igraph_vertex_disjoint_paths(argc, argv);
  } else if (!strcasecmp(base, "igraph_adhesion")) {
    return shell_igraph_adhesion(argc, argv);
  } else if (!strcasecmp(base, "igraph_cohesion")) {
    return shell_igraph_cohesion(argc, argv);
  } else if (!strcasecmp(base, "igraph_coreness")) {
    return shell_igraph_coreness(argc, argv);
  } else if (!strcasecmp(base, "igraph_isoclass")) {
    return shell_igraph_isoclass(argc, argv);
  } else if (!strcasecmp(base, "igraph_isomorphic")) {
    return shell_igraph_isomorphic(argc, argv);
  } else if (!strcasecmp(base, "igraph_isoclass_subgraph")) {
    return shell_igraph_isoclass_subgraph(argc, argv);
  } else if (!strcasecmp(base, "igraph_isoclass_create")) {
    return shell_igraph_isoclass_create(argc, argv);
  } else if (!strcasecmp(base, "igraph_isomorphic_vf2")) {
    return shell_igraph_isomorphic_vf2(argc, argv);
  } else if (!strcasecmp(base, "igraph_count_isomorphisms_vf2")) {
    return shell_igraph_count_isomorphisms_vf2(argc, argv);
  } else if (!strcasecmp(base, "igraph_get_isomorphisms_vf2")) {
    return shell_igraph_get_isomorphisms_vf2(argc, argv);
  } else if (!strcasecmp(base, "igraph_subisomorphic_vf2")) {
    return shell_igraph_subisomorphic_vf2(argc, argv);
  } else if (!strcasecmp(base, "igraph_count_subisomorphisms_vf2")) {
    return shell_igraph_count_subisomorphisms_vf2(argc, argv);
  } else if (!strcasecmp(base, "igraph_get_subisomorphisms_vf2")) {
    return shell_igraph_get_subisomorphisms_vf2(argc, argv);
  } else if (!strcasecmp(base, "igraph_isomorphic_34")) {
    return shell_igraph_isomorphic_34(argc, argv);
  } else if (!strcasecmp(base, "igraph_canonical_permutation")) {
    return shell_igraph_canonical_permutation(argc, argv);
  } else if (!strcasecmp(base, "igraph_permute_vertices")) {
    return shell_igraph_permute_vertices(argc, argv);
  } else if (!strcasecmp(base, "igraph_isomorphic_bliss")) {
    return shell_igraph_isomorphic_bliss(argc, argv);
  } else if (!strcasecmp(base, "igraph_automorphisms")) {
    return shell_igraph_automorphisms(argc, argv);
  } else if (!strcasecmp(base, "igraph_running_mean")) {
    return shell_igraph_running_mean(argc, argv);
  } else if (!strcasecmp(base, "igraph_random_sample")) {
    return shell_igraph_random_sample(argc, argv);
  } else if (!strcasecmp(base, "igraph_convex_hull")) {
    return shell_igraph_convex_hull(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_d")) {
    return shell_igraph_revolver_ml_d(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_probs_d")) {
    return shell_igraph_revolver_probs_d(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_de")) {
    return shell_igraph_revolver_ml_de(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_probs_de")) {
    return shell_igraph_revolver_probs_de(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_ade")) {
    return shell_igraph_revolver_ml_ade(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_probs_ade")) {
    return shell_igraph_revolver_probs_ade(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_f")) {
    return shell_igraph_revolver_ml_f(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_df")) {
    return shell_igraph_revolver_ml_df(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_l")) {
    return shell_igraph_revolver_ml_l(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_ad")) {
    return shell_igraph_revolver_ml_ad(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_probs_ad")) {
    return shell_igraph_revolver_probs_ad(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_D_alpha")) {
    return shell_igraph_revolver_ml_D_alpha(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_D_alpha_a")) {
    return shell_igraph_revolver_ml_D_alpha_a(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_DE_alpha_a")) {
    return shell_igraph_revolver_ml_DE_alpha_a(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_AD_alpha_a_beta")) {
    return shell_igraph_revolver_ml_AD_alpha_a_beta(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_AD_dpareto")) {
    return shell_igraph_revolver_ml_AD_dpareto(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_AD_dpareto_eval")) {
    return shell_igraph_revolver_ml_AD_dpareto_eval(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_ADE_alpha_a_beta")) {
    return shell_igraph_revolver_ml_ADE_alpha_a_beta(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_ADE_dpareto")) {
    return shell_igraph_revolver_ml_ADE_dpareto(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_ADE_dpareto_eval")) {
    return shell_igraph_revolver_ml_ADE_dpareto_eval(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_ml_ADE_dpareto_evalf")) {
    return shell_igraph_revolver_ml_ADE_dpareto_evalf(argc, argv);
  } else if (!strcasecmp(base, "igraph_revolver_probs_ADE_dpareto")) {
    return shell_igraph_revolver_probs_ADE_dpareto(argc, argv);
  } else if (!strcasecmp(base, "igraph_convergence_degree")) {
    return shell_igraph_convergence_degree(argc, argv);
  } else {
    printf("Unknown function, exiting\n");
  }

  shell_igraph_usage(argc, argv);
  return 0;

}

/* The functions themselves at last */

/*-------------------------------------------/
/ igraph_empty                               /
/-------------------------------------------*/
void shell_igraph_empty_usage(char **argv) {
  printf("%s --graph=<graph> --n=<n> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_empty(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t n=0;
  igraph_bool_t directed=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n",required_argument,0,1 },
                                   { "directed",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));
  shell_seen[1]=2;
  shell_seen[2]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 2: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 3:
      shell_igraph_empty_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_empty(&graph, n, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_add_edges                           /
/-------------------------------------------*/
void shell_igraph_add_edges_usage(char **argv) {
  printf("%s --graph=<graph> --graph-out=<graph-out> --edges=<edges>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_add_edges(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t edges;

  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "graph-out",required_argument,0,1 },
                                   { "edges",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* graph-out */
      shell_arg_graph=strdup(optarg);
      break;
    case 2: /* edges */
      shell_read_vector(&edges, optarg);
      break;
    case 3:
      shell_igraph_add_edges_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_add_edges(&graph, &edges, 0);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_vector_destroy(&edges);

  return 0;
}

/*-------------------------------------------/
/ igraph_add_vertices                        /
/-------------------------------------------*/
void shell_igraph_add_vertices_usage(char **argv) {
  printf("%s --graph=<graph> --graph-out=<graph-out> --nv=<nv>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_add_vertices(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t nv;

  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "graph-out",required_argument,0,1 },
                                   { "nv",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* graph-out */
      shell_arg_graph=strdup(optarg);
      break;
    case 2: /* nv */
      shell_read_integer(&nv, optarg);
      break;
    case 3:
      shell_igraph_add_vertices_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_add_vertices(&graph, nv, 0);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_delete_edges                        /
/-------------------------------------------*/
void shell_igraph_delete_edges_usage(char **argv) {
  printf("%s --graph=<graph> --graph-out=<graph-out> --edges=<edges>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_delete_edges(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t v_edges; igraph_es_t edges;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "graph-out",required_argument,0,1 },
                                   { "edges",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* graph-out */
      shell_arg_graph=strdup(optarg);
      break;
    case 2: /* edges */
      shell_read_vector(&v_edges, optarg); igraph_es_vector(&edges, &v_edges);
      break;
    case 3:
      shell_igraph_delete_edges_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_delete_edges(&graph, edges);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  if (!igraph_es_is_all(&edges)) { igraph_es_destroy(&edges); }

  return 0;
}

/*-------------------------------------------/
/ igraph_delete_vertices                     /
/-------------------------------------------*/
void shell_igraph_delete_vertices_usage(char **argv) {
  printf("%s --graph=<graph> --graph-out=<graph-out> --vertices=<vertices>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_delete_vertices(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t v_vertices; igraph_vs_t vertices;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "graph-out",required_argument,0,1 },
                                   { "vertices",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* graph-out */
      shell_arg_graph=strdup(optarg);
      break;
    case 2: /* vertices */
      shell_read_vector(&v_vertices, optarg); igraph_vs_vector(&vertices, &v_vertices);
      break;
    case 3:
      shell_igraph_delete_vertices_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_delete_vertices(&graph, vertices);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  if (!igraph_vs_is_all(&vertices)) { igraph_vector_destroy(&v_vertices); }

  return 0;
}

/*-------------------------------------------/
/ igraph_vcount                              /
/-------------------------------------------*/
void shell_igraph_vcount_usage(char **argv) {
  printf("%s --graph=<graph>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_vcount(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t shell_result;
  char *shell_arg_shell_result="-";

  int shell_seen[1];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "help",no_argument,0,1 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 1*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1:
      shell_igraph_vcount_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<1; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_vcount(&graph);

  /* Write the result */
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_ecount                              /
/-------------------------------------------*/
void shell_igraph_ecount_usage(char **argv) {
  printf("%s --graph=<graph>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_ecount(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t shell_result;
  char *shell_arg_shell_result="-";

  int shell_seen[1];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "help",no_argument,0,1 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 1*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1:
      shell_igraph_ecount_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<1; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_ecount(&graph);

  /* Write the result */
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_neighbors                           /
/-------------------------------------------*/
void shell_igraph_neighbors_usage(char **argv) {
  printf("%s --graph=<graph> --neis=<neis> --vid=<vid> --mode=<mode>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_neighbors(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t neis;
  igraph_integer_t vid;
  igraph_neimode_t mode=IGRAPH_ALL;
  char* shell_arg_neis=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "neis",required_argument,0,1 },
                                   { "vid",required_argument,0,2 },
                                   { "mode",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* neis */
      shell_arg_neis=strdup(optarg); 
  igraph_vector_init(&neis, 0);
      break;
    case 2: /* vid */
      shell_read_integer(&vid, optarg);
      break;
    case 3: /* mode */
      shell_read_enum(&mode, optarg, "out", 1, "in", 2, "all", 3, "total", 3, 0);
      break;
    case 4:
      shell_igraph_neighbors_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_neighbors(&graph, &neis, vid, mode);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&neis, shell_arg_neis); 
  igraph_vector_destroy(&neis);

  return 0;
}

/*-------------------------------------------/
/ igraph_is_directed                         /
/-------------------------------------------*/
void shell_igraph_is_directed_usage(char **argv) {
  printf("%s --graph=<graph>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_is_directed(int argc, char **argv) {

  igraph_t graph;
  igraph_bool_t shell_result;
  char *shell_arg_shell_result="-";

  int shell_seen[1];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "help",no_argument,0,1 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 1*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1:
      shell_igraph_is_directed_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<1; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_is_directed(&graph);

  /* Write the result */
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_degree                              /
/-------------------------------------------*/
void shell_igraph_degree_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --vids=<vids> --mode=<mode> --loops=<loops>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_degree(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t res;
  igraph_vector_t v_vids; igraph_vs_t vids=igraph_vss_all();
  igraph_neimode_t mode=IGRAPH_ALL;
  igraph_bool_t loops;
  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "vids",required_argument,0,2 },
                                   { "mode",required_argument,0,3 },
                                   { "loops",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_vector_init(&res, 0);
      break;
    case 2: /* vids */
      shell_read_vector(&v_vids, optarg); igraph_vs_vector(&vids, &v_vids);
      break;
    case 3: /* mode */
      shell_read_enum(&mode, optarg, "out", 1, "in", 2, "all", 3, "total", 3, 0);
      break;
    case 4: /* loops */
      shell_read_boolean(&loops, optarg);
      break;
    case 5:
      shell_igraph_degree_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_degree(&graph, &res, vids, mode, loops);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&res, shell_arg_res); 
  igraph_vector_destroy(&res);
  if (!igraph_vs_is_all(&vids)) { igraph_vector_destroy(&v_vids); }

  return 0;
}

/*-------------------------------------------/
/ igraph_edge                                /
/-------------------------------------------*/
void shell_igraph_edge_usage(char **argv) {
  printf("%s --graph=<graph> --eid=<eid> --from=<from> --to=<to>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_edge(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t eid;
  igraph_integer_t from;
  igraph_integer_t to;
  char* shell_arg_from=0;
  char* shell_arg_to=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "eid",required_argument,0,1 },
                                   { "from",required_argument,0,2 },
                                   { "to",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* eid */
      shell_read_integer(&eid, optarg);
      break;
    case 2: /* from */
      shell_arg_from=strdup(optarg);
      break;
    case 3: /* to */
      shell_arg_to=strdup(optarg);
      break;
    case 4:
      shell_igraph_edge_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_edge(&graph, eid, &from, &to);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_integer(from, shell_arg_from);
  shell_write_integer(to, shell_arg_to);

  return 0;
}

/*-------------------------------------------/
/ igraph_edges                               /
/-------------------------------------------*/
void shell_igraph_edges_usage(char **argv) {
  printf("%s --graph=<graph> --eids=<eids> --edges=<edges>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_edges(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t v_eids; igraph_es_t eids;
  igraph_vector_t edges;
  char* shell_arg_edges=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "eids",required_argument,0,1 },
                                   { "edges",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* eids */
      shell_read_vector(&v_eids, optarg); igraph_es_vector(&eids, &v_eids);
      break;
    case 2: /* edges */
      shell_arg_edges=strdup(optarg); 
  igraph_vector_init(&edges, 0);
      break;
    case 3:
      shell_igraph_edges_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_edges(&graph, eids, &edges);

  /* Write the result */
  igraph_destroy(&graph);
  if (!igraph_es_is_all(&eids)) { igraph_es_destroy(&eids); }
  shell_write_vector(&edges, shell_arg_edges); 
  igraph_vector_destroy(&edges);

  return 0;
}

/*-------------------------------------------/
/ igraph_get_eid                             /
/-------------------------------------------*/
void shell_igraph_get_eid_usage(char **argv) {
  printf("%s --graph=<graph> --eid=<eid> --from=<from> --to=<to> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_get_eid(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t eid;
  igraph_integer_t from;
  igraph_integer_t to;
  igraph_bool_t directed=1;
  char* shell_arg_eid=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "eid",required_argument,0,1 },
                                   { "from",required_argument,0,2 },
                                   { "to",required_argument,0,3 },
                                   { "directed",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));
  shell_seen[4]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* eid */
      shell_arg_eid=strdup(optarg);
      break;
    case 2: /* from */
      shell_read_integer(&from, optarg);
      break;
    case 3: /* to */
      shell_read_integer(&to, optarg);
      break;
    case 4: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 5:
      shell_igraph_get_eid_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_get_eid(&graph, &eid, from, to, directed);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_integer(eid, shell_arg_eid);

  return 0;
}

/*-------------------------------------------/
/ igraph_get_eids                            /
/-------------------------------------------*/
void shell_igraph_get_eids_usage(char **argv) {
  printf("%s --graph=<graph> --eids=<eids> --pairs=<pairs> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_get_eids(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t eids;
  igraph_vector_t pairs;
  igraph_bool_t directed=1;
  char* shell_arg_eids=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "eids",required_argument,0,1 },
                                   { "pairs",required_argument,0,2 },
                                   { "directed",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* eids */
      shell_arg_eids=strdup(optarg); 
  igraph_vector_init(&eids, 0);
      break;
    case 2: /* pairs */
      shell_read_vector(&pairs, optarg);
      break;
    case 3: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 4:
      shell_igraph_get_eids_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_get_eids(&graph, &eids, &pairs, directed);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&eids, shell_arg_eids); 
  igraph_vector_destroy(&eids);
  igraph_vector_destroy(&pairs);

  return 0;
}

/*-------------------------------------------/
/ igraph_adjacent                            /
/-------------------------------------------*/
void shell_igraph_adjacent_usage(char **argv) {
  printf("%s --graph=<graph> --eids=<eids> --vid=<vid> --mode=<mode>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_adjacent(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t eids;
  igraph_integer_t vid;
  igraph_neimode_t mode=IGRAPH_ALL;
  char* shell_arg_eids=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "eids",required_argument,0,1 },
                                   { "vid",required_argument,0,2 },
                                   { "mode",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* eids */
      shell_arg_eids=strdup(optarg); 
  igraph_vector_init(&eids, 0);
      break;
    case 2: /* vid */
      shell_read_integer(&vid, optarg);
      break;
    case 3: /* mode */
      shell_read_enum(&mode, optarg, "out", 1, "in", 2, "all", 3, "total", 3, 0);
      break;
    case 4:
      shell_igraph_adjacent_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_adjacent(&graph, &eids, vid, mode);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&eids, shell_arg_eids); 
  igraph_vector_destroy(&eids);

  return 0;
}

/*-------------------------------------------/
/ igraph_create                              /
/-------------------------------------------*/
void shell_igraph_create_usage(char **argv) {
  printf("%s --graph=<graph> --edges=<edges> --n=<n> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_create(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t edges;
  igraph_integer_t n=0;
  igraph_bool_t directed=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "edges",required_argument,0,1 },
                                   { "n",required_argument,0,2 },
                                   { "directed",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* edges */
      shell_read_vector(&edges, optarg);
      break;
    case 2: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 3: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 4:
      shell_igraph_create_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_create(&graph, &edges, n, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_vector_destroy(&edges);

  return 0;
}

/*-------------------------------------------/
/ igraph_adjacency                           /
/-------------------------------------------*/
void shell_igraph_adjacency_usage(char **argv) {
  printf("%s --graph=<graph> --adjmatrix=<adjmatrix> --mode=<mode>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_adjacency(int argc, char **argv) {

  igraph_t graph;
  igraph_matrix_t adjmatrix;
  igraph_adjacency_t mode=IGRAPH_ADJ_DIRECTED;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "adjmatrix",required_argument,0,1 },
                                   { "mode",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));
  shell_seen[2]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* adjmatrix */
      shell_read_matrix(&adjmatrix, optarg);
      break;
    case 2: /* mode */
      shell_read_enum(&mode, optarg, "directed", 0, "undirected", 1, 
  "max", 1, "upper", 2, "lower", 3, "min", 4, "plus", 5, 0);
      break;
    case 3:
      shell_igraph_adjacency_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_adjacency(&graph, &adjmatrix, mode);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_matrix_destroy(&adjmatrix);

  return 0;
}

/*-------------------------------------------/
/ igraph_star                                /
/-------------------------------------------*/
void shell_igraph_star_usage(char **argv) {
  printf("%s --graph=<graph> --n=<n> --mode=<mode> --center=<center>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_star(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t n;
  igraph_star_mode_t mode=IGRAPH_STAR_OUT;
  igraph_integer_t center=0;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n",required_argument,0,1 },
                                   { "mode",required_argument,0,2 },
                                   { "center",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 2: /* mode */
      shell_read_enum(&mode, optarg, "out", 0, "in", 1, "undirected", 2, 0);
      break;
    case 3: /* center */
      shell_read_integer(&center, optarg);
      break;
    case 4:
      shell_igraph_star_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_star(&graph, n, mode, center);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_lattice                             /
/-------------------------------------------*/
void shell_igraph_lattice_usage(char **argv) {
  printf("%s --graph=<graph> --dimvector=<dimvector> --nei=<nei> --directed=<directed> --mutual=<mutual> --circular=<circular>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_lattice(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t dimvector;
  igraph_integer_t nei=1;
  igraph_bool_t directed=0;
  igraph_bool_t mutual=0;
  igraph_bool_t circular=0;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[6];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "dimvector",required_argument,0,1 },
                                   { "nei",required_argument,0,2 },
                                   { "directed",required_argument,0,3 },
                                   { "mutual",required_argument,0,4 },
                                   { "circular",required_argument,0,5 },
                                   { "help",no_argument,0,6 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 6*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  shell_seen[4]=2;
  shell_seen[5]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* dimvector */
      shell_read_vector(&dimvector, optarg);
      break;
    case 2: /* nei */
      shell_read_integer(&nei, optarg);
      break;
    case 3: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 4: /* mutual */
      shell_read_boolean(&mutual, optarg);
      break;
    case 5: /* circular */
      shell_read_boolean(&circular, optarg);
      break;
    case 6:
      shell_igraph_lattice_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<6; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_lattice(&graph, &dimvector, nei, directed, mutual, circular);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_vector_destroy(&dimvector);

  return 0;
}

/*-------------------------------------------/
/ igraph_ring                                /
/-------------------------------------------*/
void shell_igraph_ring_usage(char **argv) {
  printf("%s --graph=<graph> --n=<n> --directed=<directed> --mutual=<mutual> --circular=<circular>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_ring(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t n;
  igraph_bool_t directed=0;
  igraph_bool_t mutual=0;
  igraph_bool_t circular=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n",required_argument,0,1 },
                                   { "directed",required_argument,0,2 },
                                   { "mutual",required_argument,0,3 },
                                   { "circular",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  shell_seen[4]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 2: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 3: /* mutual */
      shell_read_boolean(&mutual, optarg);
      break;
    case 4: /* circular */
      shell_read_boolean(&circular, optarg);
      break;
    case 5:
      shell_igraph_ring_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_ring(&graph, n, directed, mutual, circular);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_tree                                /
/-------------------------------------------*/
void shell_igraph_tree_usage(char **argv) {
  printf("%s --graph=<graph> --n=<n> --children=<children> --type=<type>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_tree(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t n;
  igraph_integer_t children=2;
  igraph_tree_mode_t type=IGRAPH_TREE_OUT;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n",required_argument,0,1 },
                                   { "children",required_argument,0,2 },
                                   { "type",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 2: /* children */
      shell_read_integer(&children, optarg);
      break;
    case 3: /* type */
      shell_read_enum(&type, optarg, "out", 0, "in", 1, "undirected", 2, 0);
      break;
    case 4:
      shell_igraph_tree_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_tree(&graph, n, children, type);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_full                                /
/-------------------------------------------*/
void shell_igraph_full_usage(char **argv) {
  printf("%s --graph=<graph> --n=<n> --directed=<directed> --loops=<loops>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_full(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t n;
  igraph_bool_t directed=0;
  igraph_bool_t loops=0;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n",required_argument,0,1 },
                                   { "directed",required_argument,0,2 },
                                   { "loops",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 2: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 3: /* loops */
      shell_read_boolean(&loops, optarg);
      break;
    case 4:
      shell_igraph_full_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_full(&graph, n, directed, loops);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_full_citation                       /
/-------------------------------------------*/
void shell_igraph_full_citation_usage(char **argv) {
  printf("%s --graph=<graph> --n=<n> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_full_citation(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t n;
  igraph_bool_t directed=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n",required_argument,0,1 },
                                   { "directed",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));
  shell_seen[2]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 2: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 3:
      shell_igraph_full_citation_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_full_citation(&graph, n, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_atlas                               /
/-------------------------------------------*/
void shell_igraph_atlas_usage(char **argv) {
  printf("%s --graph=<graph> --number=<number>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_atlas(int argc, char **argv) {

  igraph_t graph;
  int number=0;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[2];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "number",required_argument,0,1 },
                                   { "help",no_argument,0,2 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 2*sizeof(int));
  shell_seen[1]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* number */
      shell_read_int(&number, optarg);
      break;
    case 2:
      shell_igraph_atlas_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<2; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_atlas(&graph, number);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_extended_chordal_ring               /
/-------------------------------------------*/
void shell_igraph_extended_chordal_ring_usage(char **argv) {
  printf("%s --graph=<graph> --nodes=<nodes> --W=<W>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_extended_chordal_ring(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t nodes;
  igraph_matrix_t W;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "nodes",required_argument,0,1 },
                                   { "W",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* nodes */
      shell_read_integer(&nodes, optarg);
      break;
    case 2: /* W */
      shell_read_matrix(&W, optarg);
      break;
    case 3:
      shell_igraph_extended_chordal_ring_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_extended_chordal_ring(&graph, nodes, &W);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_matrix_destroy(&W);

  return 0;
}

/*-------------------------------------------/
/ igraph_connect_neighborhood                /
/-------------------------------------------*/
void shell_igraph_connect_neighborhood_usage(char **argv) {
  printf("%s --graph=<graph> --graph-out=<graph-out> --order=<order> --mode=<mode>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_connect_neighborhood(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t order=2;
  igraph_neimode_t mode=IGRAPH_ALL;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "graph-out",required_argument,0,1 },
                                   { "order",required_argument,0,2 },
                                   { "mode",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* graph-out */
      shell_arg_graph=strdup(optarg);
      break;
    case 2: /* order */
      shell_read_integer(&order, optarg);
      break;
    case 3: /* mode */
      shell_read_enum(&mode, optarg, "out", 1, "in", 2, "all", 3, "total", 3, 0);
      break;
    case 4:
      shell_igraph_connect_neighborhood_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_connect_neighborhood(&graph, order, mode);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_linegraph                           /
/-------------------------------------------*/
void shell_igraph_linegraph_usage(char **argv) {
  printf("%s --graph=<graph> --linegraph=<linegraph>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_linegraph(int argc, char **argv) {

  igraph_t graph;
  igraph_t linegraph;
  char* shell_arg_linegraph=0;
  int shell_result;


  int shell_seen[2];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "linegraph",required_argument,0,1 },
                                   { "help",no_argument,0,2 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 2*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* linegraph */
      shell_arg_linegraph=strdup(optarg);
      break;
    case 2:
      shell_igraph_linegraph_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<2; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_linegraph(&graph, &linegraph);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_graph(&linegraph, shell_arg_linegraph); 
  igraph_destroy(&linegraph);

  return 0;
}

/*-------------------------------------------/
/ igraph_de_bruijn                           /
/-------------------------------------------*/
void shell_igraph_de_bruijn_usage(char **argv) {
  printf("%s --graph=<graph> --m=<m> --n=<n>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_de_bruijn(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t m;
  igraph_integer_t n;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "m",required_argument,0,1 },
                                   { "n",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* m */
      shell_read_integer(&m, optarg);
      break;
    case 2: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 3:
      shell_igraph_de_bruijn_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_de_bruijn(&graph, m, n);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_kautz                               /
/-------------------------------------------*/
void shell_igraph_kautz_usage(char **argv) {
  printf("%s --graph=<graph> --m=<m> --n=<n>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_kautz(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t m;
  igraph_integer_t n;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "m",required_argument,0,1 },
                                   { "n",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* m */
      shell_read_integer(&m, optarg);
      break;
    case 2: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 3:
      shell_igraph_kautz_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_kautz(&graph, m, n);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_famous                              /
/-------------------------------------------*/
void shell_igraph_famous_usage(char **argv) {
  printf("%s --graph=<graph> --name=<name>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_famous(int argc, char **argv) {

  igraph_t graph;
  char * name="";
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[2];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "name",required_argument,0,1 },
                                   { "help",no_argument,0,2 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 2*sizeof(int));
  shell_seen[1]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* name */
      name=strdup(optarg);
      break;
    case 2:
      shell_igraph_famous_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<2; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_famous(&graph, name);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  free(name);

  return 0;
}

/*-------------------------------------------/
/ igraph_lcf_vector                          /
/-------------------------------------------*/
void shell_igraph_lcf_vector_usage(char **argv) {
  printf("%s --graph=<graph> --n=<n> --shifts=<shifts> --repeats=<repeats>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_lcf_vector(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t n;
  igraph_vector_t shifts;
  igraph_integer_t repeats=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n",required_argument,0,1 },
                                   { "shifts",required_argument,0,2 },
                                   { "repeats",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 2: /* shifts */
      shell_read_vector(&shifts, optarg);
      break;
    case 3: /* repeats */
      shell_read_integer(&repeats, optarg);
      break;
    case 4:
      shell_igraph_lcf_vector_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_lcf_vector(&graph, n, &shifts, repeats);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_vector_destroy(&shifts);

  return 0;
}

/*-------------------------------------------/
/ igraph_adjlist                             /
/-------------------------------------------*/
void shell_igraph_adjlist_usage(char **argv) {
  printf("%s --graph=<graph> --directed=<directed> --duplicate=<duplicate>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_adjlist(int argc, char **argv) {

  igraph_t graph;

  igraph_bool_t directed=1;
  igraph_bool_t duplicate=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "directed",required_argument,0,1 },
                                   { "duplicate",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));
  shell_seen[1]=2;
  shell_seen[2]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 2: /* duplicate */
      shell_read_boolean(&duplicate, optarg);
      break;
    case 3:
      shell_igraph_adjlist_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_adjlist(&graph, adjlist, directed, duplicate);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_full_bipartite                      /
/-------------------------------------------*/
void shell_igraph_full_bipartite_usage(char **argv) {
  printf("%s --graph=<graph> --n1=<n1> --n2=<n2> --directed=<directed> --mode=<mode>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_full_bipartite(int argc, char **argv) {

  igraph_t graph;

  igraph_integer_t n1;
  igraph_integer_t n2;
  igraph_bool_t directed=0;
  igraph_neimode_t mode=IGRAPH_ALL;
  char* shell_arg_graph=0;
  char* shell_arg_types=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n1",required_argument,0,1 },
                                   { "n2",required_argument,0,2 },
                                   { "directed",required_argument,0,3 },
                                   { "mode",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));
  shell_seen[3]=2;
  shell_seen[4]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n1 */
      shell_read_integer(&n1, optarg);
      break;
    case 2: /* n2 */
      shell_read_integer(&n2, optarg);
      break;
    case 3: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 4: /* mode */
      shell_read_enum(&mode, optarg, "out", 1, "in", 2, "all", 3, "total", 3, 0);
      break;
    case 5:
      shell_igraph_full_bipartite_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_full_bipartite(&graph, types, n1, n2, directed, mode);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_barabasi_game                       /
/-------------------------------------------*/
void shell_igraph_barabasi_game_usage(char **argv) {
  printf("%s --graph=<graph> --n=<n> --m=<m> --outseq=<outseq> --outpref=<outpref> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_barabasi_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t n;
  igraph_integer_t m=1;
  igraph_vector_t v_outseq; igraph_vector_t *outseq=0;
  igraph_bool_t outpref=0;
  igraph_bool_t directed=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[6];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n",required_argument,0,1 },
                                   { "m",required_argument,0,2 },
                                   { "outseq",required_argument,0,3 },
                                   { "outpref",required_argument,0,4 },
                                   { "directed",required_argument,0,5 },
                                   { "help",no_argument,0,6 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 6*sizeof(int));
  shell_seen[2]=2;
  shell_seen[4]=2;
  shell_seen[5]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 2: /* m */
      shell_read_integer(&m, optarg);
      break;
    case 3: /* outseq */
      outseq=&v_outseq; shell_read_vector(outseq, optarg);
      break;
    case 4: /* outpref */
      shell_read_boolean(&outpref, optarg);
      break;
    case 5: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 6:
      shell_igraph_barabasi_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<6; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_barabasi_game(&graph, n, m, outseq, outpref, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  if (outseq) { igraph_vector_destroy(outseq); }

  return 0;
}

/*-------------------------------------------/
/ igraph_nonlinear_barabasi_game             /
/-------------------------------------------*/
void shell_igraph_nonlinear_barabasi_game_usage(char **argv) {
  printf("%s --graph=<graph> --n=<n> --power=<power> --m=<m> --outseq=<outseq> --outpref=<outpref> --zeroappeal=<zeroappeal> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_nonlinear_barabasi_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t n;
  igraph_real_t power=1.0;
  igraph_integer_t m=1;
  igraph_vector_t v_outseq; igraph_vector_t *outseq=0;
  igraph_bool_t outpref=0;
  igraph_real_t zeroappeal=1.0;
  igraph_bool_t directed=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[8];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n",required_argument,0,1 },
                                   { "power",required_argument,0,2 },
                                   { "m",required_argument,0,3 },
                                   { "outseq",required_argument,0,4 },
                                   { "outpref",required_argument,0,5 },
                                   { "zeroappeal",required_argument,0,6 },
                                   { "directed",required_argument,0,7 },
                                   { "help",no_argument,0,8 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 8*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  shell_seen[5]=2;
  shell_seen[6]=2;
  shell_seen[7]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 2: /* power */
      shell_read_real(&power, optarg);
      break;
    case 3: /* m */
      shell_read_integer(&m, optarg);
      break;
    case 4: /* outseq */
      outseq=&v_outseq; shell_read_vector(outseq, optarg);
      break;
    case 5: /* outpref */
      shell_read_boolean(&outpref, optarg);
      break;
    case 6: /* zeroappeal */
      shell_read_real(&zeroappeal, optarg);
      break;
    case 7: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 8:
      shell_igraph_nonlinear_barabasi_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<8; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_nonlinear_barabasi_game(&graph, n, power, m, outseq, outpref, zeroappeal, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  if (outseq) { igraph_vector_destroy(outseq); }

  return 0;
}

/*-------------------------------------------/
/ igraph_erdos_renyi_game_gnp                /
/-------------------------------------------*/
void shell_igraph_erdos_renyi_game_gnp_usage(char **argv) {
  printf("%s --graph=<graph> --n=<n> --p=<p> --directed=<directed> --loops=<loops>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_erdos_renyi_game_gnp(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t n;
  igraph_real_t p;
  igraph_bool_t directed=0;
  igraph_bool_t loops=0;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n",required_argument,0,1 },
                                   { "p",required_argument,0,2 },
                                   { "directed",required_argument,0,3 },
                                   { "loops",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));
  shell_seen[3]=2;
  shell_seen[4]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 2: /* p */
      shell_read_real(&p, optarg);
      break;
    case 3: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 4: /* loops */
      shell_read_boolean(&loops, optarg);
      break;
    case 5:
      shell_igraph_erdos_renyi_game_gnp_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_erdos_renyi_game_gnp(&graph, n, p, directed, loops);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_erdos_renyi_game_gnm                /
/-------------------------------------------*/
void shell_igraph_erdos_renyi_game_gnm_usage(char **argv) {
  printf("%s --graph=<graph> --n=<n> --m=<m> --directed=<directed> --loops=<loops>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_erdos_renyi_game_gnm(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t n;
  igraph_real_t m;
  igraph_bool_t directed=0;
  igraph_bool_t loops=0;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n",required_argument,0,1 },
                                   { "m",required_argument,0,2 },
                                   { "directed",required_argument,0,3 },
                                   { "loops",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));
  shell_seen[3]=2;
  shell_seen[4]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 2: /* m */
      shell_read_real(&m, optarg);
      break;
    case 3: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 4: /* loops */
      shell_read_boolean(&loops, optarg);
      break;
    case 5:
      shell_igraph_erdos_renyi_game_gnm_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_erdos_renyi_game_gnm(&graph, n, m, directed, loops);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_degree_sequence_game                /
/-------------------------------------------*/
void shell_igraph_degree_sequence_game_usage(char **argv) {
  printf("%s --graph=<graph> --out_deg=<out_deg> --in_deg=<in_deg> --method=<method>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_degree_sequence_game(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t out_deg;
  igraph_vector_t v_in_deg; igraph_vector_t *in_deg=0;
  igraph_degseq_t method=IGRAPH_DEGSEQ_SIMPLE;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "out_deg",required_argument,0,1 },
                                   { "in_deg",required_argument,0,2 },
                                   { "method",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* out_deg */
      shell_read_vector(&out_deg, optarg);
      break;
    case 2: /* in_deg */
      in_deg=&v_in_deg; shell_read_vector(in_deg, optarg);
      break;
    case 3: /* method */
      shell_read_enum(&method, optarg, "simple", 0, 0);
      break;
    case 4:
      shell_igraph_degree_sequence_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_degree_sequence_game(&graph, &out_deg, in_deg, method);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_vector_destroy(&out_deg);
  if (in_deg) { igraph_vector_destroy(in_deg); }

  return 0;
}

/*-------------------------------------------/
/ igraph_growing_random_game                 /
/-------------------------------------------*/
void shell_igraph_growing_random_game_usage(char **argv) {
  printf("%s --graph=<graph> --n=<n> --m=<m> --directed=<directed> --citation=<citation>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_growing_random_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t n;
  igraph_integer_t m=1;
  igraph_bool_t directed=0;
  igraph_bool_t citation=0;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n",required_argument,0,1 },
                                   { "m",required_argument,0,2 },
                                   { "directed",required_argument,0,3 },
                                   { "citation",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  shell_seen[4]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 2: /* m */
      shell_read_integer(&m, optarg);
      break;
    case 3: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 4: /* citation */
      shell_read_boolean(&citation, optarg);
      break;
    case 5:
      shell_igraph_growing_random_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_growing_random_game(&graph, n, m, directed, citation);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_barabasi_aging_game                 /
/-------------------------------------------*/
void shell_igraph_barabasi_aging_game_usage(char **argv) {
  printf("%s --graph=<graph> --nodes=<nodes> --m=<m> --outseq=<outseq> --outpref=<outpref> --pa_exp=<pa_exp> --aging_exp=<aging_exp> --aging_bin=<aging_bin> --zero_deg_appeal=<zero_deg_appeal> --zero_age_appeal=<zero_age_appeal> --deg_coef=<deg_coef> --age_coef=<age_coef> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_barabasi_aging_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t nodes;
  igraph_integer_t m=1;
  igraph_vector_t v_outseq; igraph_vector_t *outseq=0;
  igraph_bool_t outpref=0;
  igraph_real_t pa_exp=1.0;
  igraph_real_t aging_exp=0.0;
  igraph_integer_t aging_bin=1;
  igraph_real_t zero_deg_appeal=1.0;
  igraph_real_t zero_age_appeal=0.0;
  igraph_real_t deg_coef=1.0;
  igraph_real_t age_coef=1.0;
  igraph_bool_t directed=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[13];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "nodes",required_argument,0,1 },
                                   { "m",required_argument,0,2 },
                                   { "outseq",required_argument,0,3 },
                                   { "outpref",required_argument,0,4 },
                                   { "pa_exp",required_argument,0,5 },
                                   { "aging_exp",required_argument,0,6 },
                                   { "aging_bin",required_argument,0,7 },
                                   { "zero_deg_appeal",required_argument,0,8 },
                                   { "zero_age_appeal",required_argument,0,9 },
                                   { "deg_coef",required_argument,0,10 },
                                   { "age_coef",required_argument,0,11 },
                                   { "directed",required_argument,0,12 },
                                   { "help",no_argument,0,13 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 13*sizeof(int));
  shell_seen[2]=2;
  shell_seen[4]=2;
  shell_seen[5]=2;
  shell_seen[6]=2;
  shell_seen[7]=2;
  shell_seen[8]=2;
  shell_seen[9]=2;
  shell_seen[10]=2;
  shell_seen[11]=2;
  shell_seen[12]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* nodes */
      shell_read_integer(&nodes, optarg);
      break;
    case 2: /* m */
      shell_read_integer(&m, optarg);
      break;
    case 3: /* outseq */
      outseq=&v_outseq; shell_read_vector(outseq, optarg);
      break;
    case 4: /* outpref */
      shell_read_boolean(&outpref, optarg);
      break;
    case 5: /* pa_exp */
      shell_read_real(&pa_exp, optarg);
      break;
    case 6: /* aging_exp */
      shell_read_real(&aging_exp, optarg);
      break;
    case 7: /* aging_bin */
      shell_read_integer(&aging_bin, optarg);
      break;
    case 8: /* zero_deg_appeal */
      shell_read_real(&zero_deg_appeal, optarg);
      break;
    case 9: /* zero_age_appeal */
      shell_read_real(&zero_age_appeal, optarg);
      break;
    case 10: /* deg_coef */
      shell_read_real(&deg_coef, optarg);
      break;
    case 11: /* age_coef */
      shell_read_real(&age_coef, optarg);
      break;
    case 12: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 13:
      shell_igraph_barabasi_aging_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<13; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_barabasi_aging_game(&graph, nodes, m, outseq, outpref, pa_exp, aging_exp, aging_bin, zero_deg_appeal, zero_age_appeal, deg_coef, age_coef, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  if (outseq) { igraph_vector_destroy(outseq); }

  return 0;
}

/*-------------------------------------------/
/ igraph_recent_degree_game                  /
/-------------------------------------------*/
void shell_igraph_recent_degree_game_usage(char **argv) {
  printf("%s --graph=<graph> --n=<n> --power=<power> --window=<window> --m=<m> --outseq=<outseq> --outpref=<outpref> --zero_appeal=<zero_appeal> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_recent_degree_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t n;
  igraph_real_t power=1.0;
  igraph_integer_t window=1;
  igraph_integer_t m=1;
  igraph_vector_t v_outseq; igraph_vector_t *outseq=0;
  igraph_bool_t outpref=0;
  igraph_real_t zero_appeal=1.0;
  igraph_bool_t directed=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[9];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "n",required_argument,0,1 },
                                   { "power",required_argument,0,2 },
                                   { "window",required_argument,0,3 },
                                   { "m",required_argument,0,4 },
                                   { "outseq",required_argument,0,5 },
                                   { "outpref",required_argument,0,6 },
                                   { "zero_appeal",required_argument,0,7 },
                                   { "directed",required_argument,0,8 },
                                   { "help",no_argument,0,9 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 9*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  shell_seen[4]=2;
  shell_seen[6]=2;
  shell_seen[7]=2;
  shell_seen[8]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 2: /* power */
      shell_read_real(&power, optarg);
      break;
    case 3: /* window */
      shell_read_integer(&window, optarg);
      break;
    case 4: /* m */
      shell_read_integer(&m, optarg);
      break;
    case 5: /* outseq */
      outseq=&v_outseq; shell_read_vector(outseq, optarg);
      break;
    case 6: /* outpref */
      shell_read_boolean(&outpref, optarg);
      break;
    case 7: /* zero_appeal */
      shell_read_real(&zero_appeal, optarg);
      break;
    case 8: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 9:
      shell_igraph_recent_degree_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<9; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_recent_degree_game(&graph, n, power, window, m, outseq, outpref, zero_appeal, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  if (outseq) { igraph_vector_destroy(outseq); }

  return 0;
}

/*-------------------------------------------/
/ igraph_recent_degree_aging_game            /
/-------------------------------------------*/
void shell_igraph_recent_degree_aging_game_usage(char **argv) {
  printf("%s --graph=<graph> --nodes=<nodes> --m=<m> --outseq=<outseq> --outpref=<outpref> --pa_exp=<pa_exp> --aging_exp=<aging_exp> --aging_bin=<aging_bin> --window=<window> --zero_appeal=<zero_appeal> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_recent_degree_aging_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t nodes;
  igraph_integer_t m=1;
  igraph_vector_t v_outseq; igraph_vector_t *outseq=0;
  igraph_bool_t outpref=0;
  igraph_real_t pa_exp=1.0;
  igraph_real_t aging_exp=0.0;
  igraph_integer_t aging_bin=1;
  igraph_integer_t window=1;
  igraph_real_t zero_appeal=1.0;
  igraph_bool_t directed=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[11];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "nodes",required_argument,0,1 },
                                   { "m",required_argument,0,2 },
                                   { "outseq",required_argument,0,3 },
                                   { "outpref",required_argument,0,4 },
                                   { "pa_exp",required_argument,0,5 },
                                   { "aging_exp",required_argument,0,6 },
                                   { "aging_bin",required_argument,0,7 },
                                   { "window",required_argument,0,8 },
                                   { "zero_appeal",required_argument,0,9 },
                                   { "directed",required_argument,0,10 },
                                   { "help",no_argument,0,11 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 11*sizeof(int));
  shell_seen[2]=2;
  shell_seen[4]=2;
  shell_seen[5]=2;
  shell_seen[6]=2;
  shell_seen[7]=2;
  shell_seen[8]=2;
  shell_seen[9]=2;
  shell_seen[10]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* nodes */
      shell_read_integer(&nodes, optarg);
      break;
    case 2: /* m */
      shell_read_integer(&m, optarg);
      break;
    case 3: /* outseq */
      outseq=&v_outseq; shell_read_vector(outseq, optarg);
      break;
    case 4: /* outpref */
      shell_read_boolean(&outpref, optarg);
      break;
    case 5: /* pa_exp */
      shell_read_real(&pa_exp, optarg);
      break;
    case 6: /* aging_exp */
      shell_read_real(&aging_exp, optarg);
      break;
    case 7: /* aging_bin */
      shell_read_integer(&aging_bin, optarg);
      break;
    case 8: /* window */
      shell_read_integer(&window, optarg);
      break;
    case 9: /* zero_appeal */
      shell_read_real(&zero_appeal, optarg);
      break;
    case 10: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 11:
      shell_igraph_recent_degree_aging_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<11; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_recent_degree_aging_game(&graph, nodes, m, outseq, outpref, pa_exp, aging_exp, aging_bin, window, zero_appeal, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  if (outseq) { igraph_vector_destroy(outseq); }

  return 0;
}

/*-------------------------------------------/
/ igraph_callaway_traits_game                /
/-------------------------------------------*/
void shell_igraph_callaway_traits_game_usage(char **argv) {
  printf("%s --graph=<graph> --nodes=<nodes> --types=<types> --edges_per_step=<edges_per_step> --type_dist=<type_dist> --pref_matrix=<pref_matrix> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_callaway_traits_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t nodes;
  igraph_integer_t types;
  igraph_integer_t edges_per_step=1;
  igraph_vector_t type_dist;
  igraph_matrix_t pref_matrix;
  igraph_bool_t directed=0;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[7];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "nodes",required_argument,0,1 },
                                   { "types",required_argument,0,2 },
                                   { "edges_per_step",required_argument,0,3 },
                                   { "type_dist",required_argument,0,4 },
                                   { "pref_matrix",required_argument,0,5 },
                                   { "directed",required_argument,0,6 },
                                   { "help",no_argument,0,7 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 7*sizeof(int));
  shell_seen[3]=2;
  shell_seen[6]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* nodes */
      shell_read_integer(&nodes, optarg);
      break;
    case 2: /* types */
      shell_read_integer(&types, optarg);
      break;
    case 3: /* edges_per_step */
      shell_read_integer(&edges_per_step, optarg);
      break;
    case 4: /* type_dist */
      shell_read_vector(&type_dist, optarg);
      break;
    case 5: /* pref_matrix */
      shell_read_matrix(&pref_matrix, optarg);
      break;
    case 6: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 7:
      shell_igraph_callaway_traits_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<7; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_callaway_traits_game(&graph, nodes, types, edges_per_step, &type_dist, &pref_matrix, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_vector_destroy(&type_dist);
  igraph_matrix_destroy(&pref_matrix);

  return 0;
}

/*-------------------------------------------/
/ igraph_establishment_game                  /
/-------------------------------------------*/
void shell_igraph_establishment_game_usage(char **argv) {
  printf("%s --graph=<graph> --nodes=<nodes> --types=<types> --k=<k> --type_dist=<type_dist> --pref_matrix=<pref_matrix> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_establishment_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t nodes;
  igraph_integer_t types;
  igraph_integer_t k=1;
  igraph_vector_t type_dist;
  igraph_matrix_t pref_matrix;
  igraph_bool_t directed=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[7];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "nodes",required_argument,0,1 },
                                   { "types",required_argument,0,2 },
                                   { "k",required_argument,0,3 },
                                   { "type_dist",required_argument,0,4 },
                                   { "pref_matrix",required_argument,0,5 },
                                   { "directed",required_argument,0,6 },
                                   { "help",no_argument,0,7 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 7*sizeof(int));
  shell_seen[3]=2;
  shell_seen[6]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* nodes */
      shell_read_integer(&nodes, optarg);
      break;
    case 2: /* types */
      shell_read_integer(&types, optarg);
      break;
    case 3: /* k */
      shell_read_integer(&k, optarg);
      break;
    case 4: /* type_dist */
      shell_read_vector(&type_dist, optarg);
      break;
    case 5: /* pref_matrix */
      shell_read_matrix(&pref_matrix, optarg);
      break;
    case 6: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 7:
      shell_igraph_establishment_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<7; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_establishment_game(&graph, nodes, types, k, &type_dist, &pref_matrix, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_vector_destroy(&type_dist);
  igraph_matrix_destroy(&pref_matrix);

  return 0;
}

/*-------------------------------------------/
/ igraph_grg_game                            /
/-------------------------------------------*/
void shell_igraph_grg_game_usage(char **argv) {
  printf("%s --graph=<graph> --nodes=<nodes> --radius=<radius> --torus=<torus> --x=<x> --y=<y>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_grg_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t nodes;
  igraph_real_t radius;
  igraph_bool_t torus=0;
  igraph_vector_t v_x; igraph_vector_t *x=0;
  igraph_vector_t v_y; igraph_vector_t *y=0;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[6];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "nodes",required_argument,0,1 },
                                   { "radius",required_argument,0,2 },
                                   { "torus",required_argument,0,3 },
                                   { "x",required_argument,0,4 },
                                   { "y",required_argument,0,5 },
                                   { "help",no_argument,0,6 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 6*sizeof(int));
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* nodes */
      shell_read_integer(&nodes, optarg);
      break;
    case 2: /* radius */
      shell_read_real(&radius, optarg);
      break;
    case 3: /* torus */
      shell_read_boolean(&torus, optarg);
      break;
    case 4: /* x */
      x=&v_x; shell_read_vector(x, optarg);
      break;
    case 5: /* y */
      y=&v_y; shell_read_vector(y, optarg);
      break;
    case 6:
      shell_igraph_grg_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<6; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_grg_game(&graph, nodes, radius, torus, x, y);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  if (x) { igraph_vector_destroy(x); }
  if (y) { igraph_vector_destroy(y); }

  return 0;
}

/*-------------------------------------------/
/ igraph_preference_game                     /
/-------------------------------------------*/
void shell_igraph_preference_game_usage(char **argv) {
  printf("%s --graph=<graph> --nodes=<nodes> --types=<types> --type_dist=<type_dist> --pref_matrix=<pref_matrix> --node_type_vec=<node_type_vec> --directed=<directed> --loops=<loops>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_preference_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t nodes;
  igraph_integer_t types;
  igraph_vector_t type_dist;
  igraph_matrix_t pref_matrix;
  igraph_vector_t node_type_vec;
  igraph_bool_t directed=0;
  igraph_bool_t loops=0;
  char* shell_arg_graph=0;
  char* shell_arg_node_type_vec=0;
  int shell_result;


  int shell_seen[8];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "nodes",required_argument,0,1 },
                                   { "types",required_argument,0,2 },
                                   { "type_dist",required_argument,0,3 },
                                   { "pref_matrix",required_argument,0,4 },
                                   { "node_type_vec",required_argument,0,5 },
                                   { "directed",required_argument,0,6 },
                                   { "loops",required_argument,0,7 },
                                   { "help",no_argument,0,8 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 8*sizeof(int));
  shell_seen[6]=2;
  shell_seen[7]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* nodes */
      shell_read_integer(&nodes, optarg);
      break;
    case 2: /* types */
      shell_read_integer(&types, optarg);
      break;
    case 3: /* type_dist */
      shell_read_vector(&type_dist, optarg);
      break;
    case 4: /* pref_matrix */
      shell_read_matrix(&pref_matrix, optarg);
      break;
    case 5: /* node_type_vec */
      shell_arg_node_type_vec=strdup(optarg); 
  igraph_vector_init(&node_type_vec, 0);
      break;
    case 6: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 7: /* loops */
      shell_read_boolean(&loops, optarg);
      break;
    case 8:
      shell_igraph_preference_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<8; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_preference_game(&graph, nodes, types, &type_dist, &pref_matrix, &node_type_vec, directed, loops);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_vector_destroy(&type_dist);
  igraph_matrix_destroy(&pref_matrix);
  shell_write_vector(&node_type_vec, shell_arg_node_type_vec); 
  igraph_vector_destroy(&node_type_vec);

  return 0;
}

/*-------------------------------------------/
/ igraph_asymmetric_preference_game          /
/-------------------------------------------*/
void shell_igraph_asymmetric_preference_game_usage(char **argv) {
  printf("%s --graph=<graph> --nodes=<nodes> --types=<types> --type_dist_matrix=<type_dist_matrix> --pref_matrix=<pref_matrix> --node_type_in_vec=<node_type_in_vec> --node_type_out_vec=<node_type_out_vec> --loops=<loops>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_asymmetric_preference_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t nodes;
  igraph_integer_t types;
  igraph_matrix_t type_dist_matrix;
  igraph_matrix_t pref_matrix;
  igraph_vector_t node_type_in_vec;
  igraph_vector_t node_type_out_vec;
  igraph_bool_t loops=0;
  char* shell_arg_graph=0;
  char* shell_arg_node_type_in_vec=0;
  char* shell_arg_node_type_out_vec=0;
  int shell_result;


  int shell_seen[8];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "nodes",required_argument,0,1 },
                                   { "types",required_argument,0,2 },
                                   { "type_dist_matrix",required_argument,0,3 },
                                   { "pref_matrix",required_argument,0,4 },
                                   { "node_type_in_vec",required_argument,0,5 },
                                   { "node_type_out_vec",required_argument,0,6 },
                                   { "loops",required_argument,0,7 },
                                   { "help",no_argument,0,8 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 8*sizeof(int));
  shell_seen[7]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* nodes */
      shell_read_integer(&nodes, optarg);
      break;
    case 2: /* types */
      shell_read_integer(&types, optarg);
      break;
    case 3: /* type_dist_matrix */
      shell_read_matrix(&type_dist_matrix, optarg);
      break;
    case 4: /* pref_matrix */
      shell_read_matrix(&pref_matrix, optarg);
      break;
    case 5: /* node_type_in_vec */
      shell_arg_node_type_in_vec=strdup(optarg); 
  igraph_vector_init(&node_type_in_vec, 0);
      break;
    case 6: /* node_type_out_vec */
      shell_arg_node_type_out_vec=strdup(optarg); 
  igraph_vector_init(&node_type_out_vec, 0);
      break;
    case 7: /* loops */
      shell_read_boolean(&loops, optarg);
      break;
    case 8:
      shell_igraph_asymmetric_preference_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<8; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_asymmetric_preference_game(&graph, nodes, types, &type_dist_matrix, &pref_matrix, &node_type_in_vec, &node_type_out_vec, loops);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_matrix_destroy(&type_dist_matrix);
  igraph_matrix_destroy(&pref_matrix);
  shell_write_vector(&node_type_in_vec, shell_arg_node_type_in_vec); 
  igraph_vector_destroy(&node_type_in_vec);
  shell_write_vector(&node_type_out_vec, shell_arg_node_type_out_vec); 
  igraph_vector_destroy(&node_type_out_vec);

  return 0;
}

/*-------------------------------------------/
/ igraph_rewire_edges                        /
/-------------------------------------------*/
void shell_igraph_rewire_edges_usage(char **argv) {
  printf("%s --graph=<graph> --graph-out=<graph-out> --prob=<prob>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_rewire_edges(int argc, char **argv) {

  igraph_t graph;
  igraph_real_t prob;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "graph-out",required_argument,0,1 },
                                   { "prob",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* graph-out */
      shell_arg_graph=strdup(optarg);
      break;
    case 2: /* prob */
      shell_read_real(&prob, optarg);
      break;
    case 3:
      shell_igraph_rewire_edges_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_rewire_edges(&graph, prob);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_watts_strogatz_game                 /
/-------------------------------------------*/
void shell_igraph_watts_strogatz_game_usage(char **argv) {
  printf("%s --graph=<graph> --dim=<dim> --size=<size> --nei=<nei> --p=<p>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_watts_strogatz_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t dim;
  igraph_integer_t size;
  igraph_integer_t nei;
  igraph_real_t p;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "dim",required_argument,0,1 },
                                   { "size",required_argument,0,2 },
                                   { "nei",required_argument,0,3 },
                                   { "p",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* dim */
      shell_read_integer(&dim, optarg);
      break;
    case 2: /* size */
      shell_read_integer(&size, optarg);
      break;
    case 3: /* nei */
      shell_read_integer(&nei, optarg);
      break;
    case 4: /* p */
      shell_read_real(&p, optarg);
      break;
    case 5:
      shell_igraph_watts_strogatz_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_watts_strogatz_game(&graph, dim, size, nei, p);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_lastcit_game                        /
/-------------------------------------------*/
void shell_igraph_lastcit_game_usage(char **argv) {
  printf("%s --graph=<graph> --nodes=<nodes> --edges_per_node=<edges_per_node> --agebins=<agebins> --preference=<preference> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_lastcit_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t nodes;
  igraph_integer_t edges_per_node=1;
  igraph_integer_t agebins=1;
  igraph_vector_t preference;
  igraph_bool_t directed=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[6];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "nodes",required_argument,0,1 },
                                   { "edges_per_node",required_argument,0,2 },
                                   { "agebins",required_argument,0,3 },
                                   { "preference",required_argument,0,4 },
                                   { "directed",required_argument,0,5 },
                                   { "help",no_argument,0,6 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 6*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  shell_seen[5]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* nodes */
      shell_read_integer(&nodes, optarg);
      break;
    case 2: /* edges_per_node */
      shell_read_integer(&edges_per_node, optarg);
      break;
    case 3: /* agebins */
      shell_read_integer(&agebins, optarg);
      break;
    case 4: /* preference */
      shell_read_vector(&preference, optarg);
      break;
    case 5: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 6:
      shell_igraph_lastcit_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<6; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_lastcit_game(&graph, nodes, edges_per_node, agebins, &preference, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_vector_destroy(&preference);

  return 0;
}

/*-------------------------------------------/
/ igraph_cited_type_game                     /
/-------------------------------------------*/
void shell_igraph_cited_type_game_usage(char **argv) {
  printf("%s --graph=<graph> --nodes=<nodes> --types=<types> --pref=<pref> --edges_per_step=<edges_per_step> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_cited_type_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t nodes;
  igraph_vector_t types;
  igraph_vector_t pref;
  igraph_integer_t edges_per_step=1;
  igraph_bool_t directed=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[6];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "nodes",required_argument,0,1 },
                                   { "types",required_argument,0,2 },
                                   { "pref",required_argument,0,3 },
                                   { "edges_per_step",required_argument,0,4 },
                                   { "directed",required_argument,0,5 },
                                   { "help",no_argument,0,6 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 6*sizeof(int));
  shell_seen[4]=2;
  shell_seen[5]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* nodes */
      shell_read_integer(&nodes, optarg);
      break;
    case 2: /* types */
      shell_read_vector(&types, optarg);
      break;
    case 3: /* pref */
      shell_read_vector(&pref, optarg);
      break;
    case 4: /* edges_per_step */
      shell_read_integer(&edges_per_step, optarg);
      break;
    case 5: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 6:
      shell_igraph_cited_type_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<6; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_cited_type_game(&graph, nodes, &types, &pref, edges_per_step, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_vector_destroy(&types);
  igraph_vector_destroy(&pref);

  return 0;
}

/*-------------------------------------------/
/ igraph_citing_cited_type_game              /
/-------------------------------------------*/
void shell_igraph_citing_cited_type_game_usage(char **argv) {
  printf("%s --graph=<graph> --nodes=<nodes> --types=<types> --pref=<pref> --edges_per_step=<edges_per_step> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_citing_cited_type_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t nodes;
  igraph_vector_t types;
  igraph_matrix_t pref;
  igraph_integer_t edges_per_step=1;
  igraph_bool_t directed=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[6];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "nodes",required_argument,0,1 },
                                   { "types",required_argument,0,2 },
                                   { "pref",required_argument,0,3 },
                                   { "edges_per_step",required_argument,0,4 },
                                   { "directed",required_argument,0,5 },
                                   { "help",no_argument,0,6 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 6*sizeof(int));
  shell_seen[4]=2;
  shell_seen[5]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* nodes */
      shell_read_integer(&nodes, optarg);
      break;
    case 2: /* types */
      shell_read_vector(&types, optarg);
      break;
    case 3: /* pref */
      shell_read_matrix(&pref, optarg);
      break;
    case 4: /* edges_per_step */
      shell_read_integer(&edges_per_step, optarg);
      break;
    case 5: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 6:
      shell_igraph_citing_cited_type_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<6; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_citing_cited_type_game(&graph, nodes, &types, &pref, edges_per_step, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);
  igraph_vector_destroy(&types);
  igraph_matrix_destroy(&pref);

  return 0;
}

/*-------------------------------------------/
/ igraph_forest_fire_game                    /
/-------------------------------------------*/
void shell_igraph_forest_fire_game_usage(char **argv) {
  printf("%s --graph=<graph> --nodes=<nodes> --fw_prob=<fw_prob> --bw_factor=<bw_factor> --ambs=<ambs> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_forest_fire_game(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t nodes;
  igraph_real_t fw_prob;
  igraph_real_t bw_factor=1;
  igraph_integer_t ambs=1;
  igraph_bool_t directed=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[6];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "nodes",required_argument,0,1 },
                                   { "fw_prob",required_argument,0,2 },
                                   { "bw_factor",required_argument,0,3 },
                                   { "ambs",required_argument,0,4 },
                                   { "directed",required_argument,0,5 },
                                   { "help",no_argument,0,6 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 6*sizeof(int));
  shell_seen[3]=2;
  shell_seen[4]=2;
  shell_seen[5]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_arg_graph=strdup(optarg);
      break;
    case 1: /* nodes */
      shell_read_integer(&nodes, optarg);
      break;
    case 2: /* fw_prob */
      shell_read_real(&fw_prob, optarg);
      break;
    case 3: /* bw_factor */
      shell_read_real(&bw_factor, optarg);
      break;
    case 4: /* ambs */
      shell_read_integer(&ambs, optarg);
      break;
    case 5: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 6:
      shell_igraph_forest_fire_game_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<6; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_forest_fire_game(&graph, nodes, fw_prob, bw_factor, ambs, directed);

  /* Write the result */
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}

/*-------------------------------------------/
/ igraph_are_connected                       /
/-------------------------------------------*/
void shell_igraph_are_connected_usage(char **argv) {
  printf("%s --graph=<graph> --v1=<v1> --v2=<v2> --res=<res>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_are_connected(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t v1;
  igraph_integer_t v2;
  igraph_bool_t res;
  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "v1",required_argument,0,1 },
                                   { "v2",required_argument,0,2 },
                                   { "res",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* v1 */
      shell_read_integer(&v1, optarg);
      break;
    case 2: /* v2 */
      shell_read_integer(&v2, optarg);
      break;
    case 3: /* res */
      
      break;
    case 4:
      shell_igraph_are_connected_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_are_connected(&graph, v1, v2, &res);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_boolean(res, shell_arg_res);

  return 0;
}

/*-------------------------------------------/
/ igraph_diameter                            /
/-------------------------------------------*/
void shell_igraph_diameter_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --from=<from> --to=<to> --path=<path> --directed=<directed> --unconnected=<unconnected>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_diameter(int argc, char **argv) {

  igraph_t graph;
  igraph_integer_t res;
  igraph_integer_t from;
  igraph_integer_t to;
  igraph_vector_t v_path; igraph_vector_t *path=0;
  igraph_bool_t directed=1;
  igraph_bool_t unconnected=1;
  char* shell_arg_res=0;
  char* shell_arg_from=0;
  char* shell_arg_to=0;
  char* shell_arg_path=0;
  int shell_result;


  int shell_seen[7];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "from",required_argument,0,2 },
                                   { "to",required_argument,0,3 },
                                   { "path",required_argument,0,4 },
                                   { "directed",required_argument,0,5 },
                                   { "unconnected",required_argument,0,6 },
                                   { "help",no_argument,0,7 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 7*sizeof(int));
  shell_seen[5]=2;
  shell_seen[6]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg);
      break;
    case 2: /* from */
      shell_arg_from=strdup(optarg);
      break;
    case 3: /* to */
      shell_arg_to=strdup(optarg);
      break;
    case 4: /* path */
      path=&v_path; igraph_vector_init(path, 0); 
  shell_arg_path=strdup(optarg);
      break;
    case 5: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 6: /* unconnected */
      shell_read_boolean(&unconnected, optarg);
      break;
    case 7:
      shell_igraph_diameter_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<7; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_diameter(&graph, &res, &from, &to, path, directed, unconnected);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_integer(res, shell_arg_res);
  shell_write_integer(from, shell_arg_from);
  shell_write_integer(to, shell_arg_to);
  if (path) { shell_write_vector(path, shell_arg_path); 
  igraph_vector_destroy(path); }

  return 0;
}

/*-------------------------------------------/
/ igraph_diameter_dijkstra                   /
/-------------------------------------------*/
void shell_igraph_diameter_dijkstra_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --from=<from> --to=<to> --path=<path> --directed=<directed> --unconnected=<unconnected>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_diameter_dijkstra(int argc, char **argv) {

  igraph_t graph;

  igraph_integer_t res;
  igraph_integer_t from;
  igraph_integer_t to;
  igraph_vector_t v_path; igraph_vector_t *path=0;
  igraph_bool_t directed=1;
  igraph_bool_t unconnected=1;
  char* shell_arg_res=0;
  char* shell_arg_from=0;
  char* shell_arg_to=0;
  char* shell_arg_path=0;
  int shell_result;


  int shell_seen[7];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "from",required_argument,0,2 },
                                   { "to",required_argument,0,3 },
                                   { "path",required_argument,0,4 },
                                   { "directed",required_argument,0,5 },
                                   { "unconnected",required_argument,0,6 },
                                   { "help",no_argument,0,7 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 7*sizeof(int));
  shell_seen[5]=2;
  shell_seen[6]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg);
      break;
    case 2: /* from */
      shell_arg_from=strdup(optarg);
      break;
    case 3: /* to */
      shell_arg_to=strdup(optarg);
      break;
    case 4: /* path */
      path=&v_path; igraph_vector_init(path, 0); 
  shell_arg_path=strdup(optarg);
      break;
    case 5: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 6: /* unconnected */
      shell_read_boolean(&unconnected, optarg);
      break;
    case 7:
      shell_igraph_diameter_dijkstra_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<7; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_diameter_dijkstra(&graph, 0, &res, &from, &to, path, directed, unconnected);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_integer(res, shell_arg_res);
  shell_write_integer(from, shell_arg_from);
  shell_write_integer(to, shell_arg_to);
  if (path) { shell_write_vector(path, shell_arg_path); 
  igraph_vector_destroy(path); }

  return 0;
}

/*-------------------------------------------/
/ igraph_minimum_spanning_tree_unweighted    /
/-------------------------------------------*/
void shell_igraph_minimum_spanning_tree_unweighted_usage(char **argv) {
  printf("%s --graph=<graph> --mst=<mst>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_minimum_spanning_tree_unweighted(int argc, char **argv) {

  igraph_t graph;
  igraph_t mst;
  char* shell_arg_mst=0;
  int shell_result;


  int shell_seen[2];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "mst",required_argument,0,1 },
                                   { "help",no_argument,0,2 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 2*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* mst */
      shell_arg_mst=strdup(optarg);
      break;
    case 2:
      shell_igraph_minimum_spanning_tree_unweighted_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<2; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_minimum_spanning_tree_unweighted(&graph, &mst);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_graph(&mst, shell_arg_mst); 
  igraph_destroy(&mst);

  return 0;
}

/*-------------------------------------------/
/ igraph_minimum_spanning_tree_prim          /
/-------------------------------------------*/
void shell_igraph_minimum_spanning_tree_prim_usage(char **argv) {
  printf("%s --graph=<graph> --mst=<mst> --weights=<weights>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_minimum_spanning_tree_prim(int argc, char **argv) {

  igraph_t graph;
  igraph_t mst;
  igraph_vector_t weights;
  char* shell_arg_mst=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "mst",required_argument,0,1 },
                                   { "weights",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* mst */
      shell_arg_mst=strdup(optarg);
      break;
    case 2: /* weights */
      shell_read_vector(&weights, optarg);
      break;
    case 3:
      shell_igraph_minimum_spanning_tree_prim_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_minimum_spanning_tree_prim(&graph, &mst, &weights);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_graph(&mst, shell_arg_mst); 
  igraph_destroy(&mst);
  igraph_vector_destroy(&weights);

  return 0;
}

/*-------------------------------------------/
/ igraph_closeness                           /
/-------------------------------------------*/
void shell_igraph_closeness_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --vids=<vids> --mode=<mode>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_closeness(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t res;
  igraph_vector_t v_vids; igraph_vs_t vids=igraph_vss_all();
  igraph_neimode_t mode=IGRAPH_OUT;

  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "vids",required_argument,0,2 },
                                   { "mode",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_vector_init(&res, 0);
      break;
    case 2: /* vids */
      shell_read_vector(&v_vids, optarg); igraph_vs_vector(&vids, &v_vids);
      break;
    case 3: /* mode */
      shell_read_enum(&mode, optarg, "out", 1, "in", 2, "all", 3, "total", 3, 0);
      break;
    case 4:
      shell_igraph_closeness_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_closeness(&graph, &res, vids, mode, 0);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&res, shell_arg_res); 
  igraph_vector_destroy(&res);
  if (!igraph_vs_is_all(&vids)) { igraph_vector_destroy(&v_vids); }

  return 0;
}

/*-------------------------------------------/
/ igraph_closeness_estimate                  /
/-------------------------------------------*/
void shell_igraph_closeness_estimate_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --vids=<vids> --mode=<mode> --cutoff=<cutoff>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_closeness_estimate(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t res;
  igraph_vector_t v_vids; igraph_vs_t vids=igraph_vss_all();
  igraph_neimode_t mode=IGRAPH_OUT;
  igraph_real_t cutoff;

  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "vids",required_argument,0,2 },
                                   { "mode",required_argument,0,3 },
                                   { "cutoff",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_vector_init(&res, 0);
      break;
    case 2: /* vids */
      shell_read_vector(&v_vids, optarg); igraph_vs_vector(&vids, &v_vids);
      break;
    case 3: /* mode */
      shell_read_enum(&mode, optarg, "out", 1, "in", 2, "all", 3, "total", 3, 0);
      break;
    case 4: /* cutoff */
      shell_read_real(&cutoff, optarg);
      break;
    case 5:
      shell_igraph_closeness_estimate_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_closeness_estimate(&graph, &res, vids, mode, cutoff, 0);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&res, shell_arg_res); 
  igraph_vector_destroy(&res);
  if (!igraph_vs_is_all(&vids)) { igraph_vector_destroy(&v_vids); }

  return 0;
}

/*-------------------------------------------/
/ igraph_shortest_paths                      /
/-------------------------------------------*/
void shell_igraph_shortest_paths_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --from=<from> --to=<to> --mode=<mode>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_shortest_paths(int argc, char **argv) {

  igraph_t graph;
  igraph_matrix_t res;
  igraph_vector_t v_from; igraph_vs_t from=igraph_vss_all();
  igraph_vector_t v_to; igraph_vs_t to=igraph_vss_all();
  igraph_neimode_t mode=IGRAPH_OUT;
  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "from",required_argument,0,2 },
                                   { "to",required_argument,0,3 },
                                   { "mode",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  shell_seen[4]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_matrix_init(&res, 0, 0);
      break;
    case 2: /* from */
      shell_read_vector(&v_from, optarg); igraph_vs_vector(&from, &v_from);
      break;
    case 3: /* to */
      shell_read_vector(&v_to, optarg); igraph_vs_vector(&to, &v_to);
      break;
    case 4: /* mode */
      shell_read_enum(&mode, optarg, "out", 1, "in", 2, "all", 3, "total", 3, 0);
      break;
    case 5:
      shell_igraph_shortest_paths_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_shortest_paths(&graph, &res, from, to, mode);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_matrix(&res, shell_arg_res); 
  igraph_matrix_destroy(&res);
  if (!igraph_vs_is_all(&from)) { igraph_vector_destroy(&v_from); }
  if (!igraph_vs_is_all(&to)) { igraph_vector_destroy(&v_to); }

  return 0;
}

/*-------------------------------------------/
/ igraph_get_shortest_paths                  /
/-------------------------------------------*/
void shell_igraph_get_shortest_paths_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --from=<from> --to=<to> --mode=<mode>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_get_shortest_paths(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_ptr_t res;
  igraph_integer_t from;
  igraph_vector_t v_to; igraph_vs_t to=igraph_vss_all();
  igraph_neimode_t mode=IGRAPH_OUT;
  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "from",required_argument,0,2 },
                                   { "to",required_argument,0,3 },
                                   { "mode",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));
  shell_seen[3]=2;
  shell_seen[4]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      igraph_vector_ptr_init(&res, 0);
      break;
    case 2: /* from */
      shell_read_integer(&from, optarg);
      break;
    case 3: /* to */
      shell_read_vector(&v_to, optarg); igraph_vs_vector(&to, &v_to);
      break;
    case 4: /* mode */
      shell_read_enum(&mode, optarg, "out", 1, "in", 2, "all", 3, "total", 3, 0);
      break;
    case 5:
      shell_igraph_get_shortest_paths_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_get_shortest_paths(&graph, &res, from, to, mode);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vectorlist(&res, shell_arg_res); 
  shell_free_vectorlist(&res); 
  igraph_vector_ptr_destroy(&res);
  if (!igraph_vs_is_all(&to)) { igraph_vector_destroy(&v_to); }

  return 0;
}

/*-------------------------------------------/
/ igraph_get_all_shortest_paths              /
/-------------------------------------------*/
void shell_igraph_get_all_shortest_paths_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --nrgeo=<nrgeo> --from=<from> --to=<to> --mode=<mode>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_get_all_shortest_paths(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_ptr_t res;
  igraph_vector_t nrgeo;
  igraph_integer_t from;
  igraph_vector_t v_to; igraph_vs_t to;
  igraph_neimode_t mode=IGRAPH_OUT;
  char* shell_arg_res=0;
  char* shell_arg_nrgeo=0;
  int shell_result;


  int shell_seen[6];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "nrgeo",required_argument,0,2 },
                                   { "from",required_argument,0,3 },
                                   { "to",required_argument,0,4 },
                                   { "mode",required_argument,0,5 },
                                   { "help",no_argument,0,6 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 6*sizeof(int));
  shell_seen[5]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      igraph_vector_ptr_init(&res, 0);
      break;
    case 2: /* nrgeo */
      shell_arg_nrgeo=strdup(optarg); 
  igraph_vector_init(&nrgeo, 0);
      break;
    case 3: /* from */
      shell_read_integer(&from, optarg);
      break;
    case 4: /* to */
      shell_read_vector(&v_to, optarg); igraph_vs_vector(&to, &v_to);
      break;
    case 5: /* mode */
      shell_read_enum(&mode, optarg, "out", 1, "in", 2, "all", 3, "total", 3, 0);
      break;
    case 6:
      shell_igraph_get_all_shortest_paths_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<6; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_get_all_shortest_paths(&graph, &res, &nrgeo, from, to, mode);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vectorlist(&res, shell_arg_res); 
  shell_free_vectorlist(&res); 
  igraph_vector_ptr_destroy(&res);
  shell_write_vector(&nrgeo, shell_arg_nrgeo); 
  igraph_vector_destroy(&nrgeo);
  if (!igraph_vs_is_all(&to)) { igraph_vector_destroy(&v_to); }

  return 0;
}

/*-------------------------------------------/
/ igraph_shortest_paths_dijkstra             /
/-------------------------------------------*/
void shell_igraph_shortest_paths_dijkstra_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --from=<from> --to=<to> --mode=<mode>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_shortest_paths_dijkstra(int argc, char **argv) {

  igraph_t graph;
  igraph_matrix_t res;
  igraph_vector_t v_from; igraph_vs_t from=igraph_vss_all();
  igraph_vector_t v_to; igraph_vs_t to=igraph_vss_all();

  igraph_neimode_t mode=IGRAPH_OUT;
  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "from",required_argument,0,2 },
                                   { "to",required_argument,0,3 },
                                   { "mode",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  shell_seen[4]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_matrix_init(&res, 0, 0);
      break;
    case 2: /* from */
      shell_read_vector(&v_from, optarg); igraph_vs_vector(&from, &v_from);
      break;
    case 3: /* to */
      shell_read_vector(&v_to, optarg); igraph_vs_vector(&to, &v_to);
      break;
    case 4: /* mode */
      shell_read_enum(&mode, optarg, "out", 1, "in", 2, "all", 3, "total", 3, 0);
      break;
    case 5:
      shell_igraph_shortest_paths_dijkstra_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_shortest_paths_dijkstra(&graph, &res, from, to, 0, mode);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_matrix(&res, shell_arg_res); 
  igraph_matrix_destroy(&res);
  if (!igraph_vs_is_all(&from)) { igraph_vector_destroy(&v_from); }
  if (!igraph_vs_is_all(&to)) { igraph_vector_destroy(&v_to); }

  return 0;
}

/*-------------------------------------------/
/ igraph_shortest_paths_bellman_ford         /
/-------------------------------------------*/
void shell_igraph_shortest_paths_bellman_ford_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --from=<from> --to=<to> --mode=<mode>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_shortest_paths_bellman_ford(int argc, char **argv) {

  igraph_t graph;
  igraph_matrix_t res;
  igraph_vector_t v_from; igraph_vs_t from=igraph_vss_all();
  igraph_vector_t v_to; igraph_vs_t to=igraph_vss_all();

  igraph_neimode_t mode=IGRAPH_OUT;
  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "from",required_argument,0,2 },
                                   { "to",required_argument,0,3 },
                                   { "mode",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  shell_seen[4]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_matrix_init(&res, 0, 0);
      break;
    case 2: /* from */
      shell_read_vector(&v_from, optarg); igraph_vs_vector(&from, &v_from);
      break;
    case 3: /* to */
      shell_read_vector(&v_to, optarg); igraph_vs_vector(&to, &v_to);
      break;
    case 4: /* mode */
      shell_read_enum(&mode, optarg, "out", 1, "in", 2, "all", 3, "total", 3, 0);
      break;
    case 5:
      shell_igraph_shortest_paths_bellman_ford_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_shortest_paths_bellman_ford(&graph, &res, from, to, 0, mode);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_matrix(&res, shell_arg_res); 
  igraph_matrix_destroy(&res);
  if (!igraph_vs_is_all(&from)) { igraph_vector_destroy(&v_from); }
  if (!igraph_vs_is_all(&to)) { igraph_vector_destroy(&v_to); }

  return 0;
}

/*-------------------------------------------/
/ igraph_shortest_paths_johnson              /
/-------------------------------------------*/
void shell_igraph_shortest_paths_johnson_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --from=<from> --to=<to>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_shortest_paths_johnson(int argc, char **argv) {

  igraph_t graph;
  igraph_matrix_t res;
  igraph_vector_t v_from; igraph_vs_t from=igraph_vss_all();
  igraph_vector_t v_to; igraph_vs_t to=igraph_vss_all();

  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "from",required_argument,0,2 },
                                   { "to",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_matrix_init(&res, 0, 0);
      break;
    case 2: /* from */
      shell_read_vector(&v_from, optarg); igraph_vs_vector(&from, &v_from);
      break;
    case 3: /* to */
      shell_read_vector(&v_to, optarg); igraph_vs_vector(&to, &v_to);
      break;
    case 4:
      shell_igraph_shortest_paths_johnson_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_shortest_paths_johnson(&graph, &res, from, to, 0);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_matrix(&res, shell_arg_res); 
  igraph_matrix_destroy(&res);
  if (!igraph_vs_is_all(&from)) { igraph_vector_destroy(&v_from); }
  if (!igraph_vs_is_all(&to)) { igraph_vector_destroy(&v_to); }

  return 0;
}

/*-------------------------------------------/
/ igraph_subcomponent                        /
/-------------------------------------------*/
void shell_igraph_subcomponent_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --vid=<vid> --mode=<mode>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_subcomponent(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t res;
  igraph_real_t vid;
  igraph_neimode_t mode=IGRAPH_ALL;
  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "vid",required_argument,0,2 },
                                   { "mode",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_vector_init(&res, 0);
      break;
    case 2: /* vid */
      shell_read_real(&vid, optarg);
      break;
    case 3: /* mode */
      shell_read_enum(&mode, optarg, "out", 1, "in", 2, "all", 3, "total", 3, 0);
      break;
    case 4:
      shell_igraph_subcomponent_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_subcomponent(&graph, &res, vid, mode);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&res, shell_arg_res); 
  igraph_vector_destroy(&res);

  return 0;
}

/*-------------------------------------------/
/ igraph_betweenness                         /
/-------------------------------------------*/
void shell_igraph_betweenness_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --vids=<vids> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_betweenness(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t res;
  igraph_vector_t v_vids; igraph_vs_t vids=igraph_vss_all();
  igraph_bool_t directed=1;

  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "vids",required_argument,0,2 },
                                   { "directed",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_vector_init(&res, 0);
      break;
    case 2: /* vids */
      shell_read_vector(&v_vids, optarg); igraph_vs_vector(&vids, &v_vids);
      break;
    case 3: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 4:
      shell_igraph_betweenness_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_betweenness(&graph, &res, vids, directed, 0);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&res, shell_arg_res); 
  igraph_vector_destroy(&res);
  if (!igraph_vs_is_all(&vids)) { igraph_vector_destroy(&v_vids); }

  return 0;
}

/*-------------------------------------------/
/ igraph_betweenness_estimate                /
/-------------------------------------------*/
void shell_igraph_betweenness_estimate_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --vids=<vids> --directed=<directed> --cutoff=<cutoff>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_betweenness_estimate(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t res;
  igraph_vector_t v_vids; igraph_vs_t vids=igraph_vss_all();
  igraph_bool_t directed=1;
  igraph_real_t cutoff;

  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[5];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "vids",required_argument,0,2 },
                                   { "directed",required_argument,0,3 },
                                   { "cutoff",required_argument,0,4 },
                                   { "help",no_argument,0,5 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 5*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_vector_init(&res, 0);
      break;
    case 2: /* vids */
      shell_read_vector(&v_vids, optarg); igraph_vs_vector(&vids, &v_vids);
      break;
    case 3: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 4: /* cutoff */
      shell_read_real(&cutoff, optarg);
      break;
    case 5:
      shell_igraph_betweenness_estimate_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<5; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_betweenness_estimate(&graph, &res, vids, directed, cutoff, 0);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&res, shell_arg_res); 
  igraph_vector_destroy(&res);
  if (!igraph_vs_is_all(&vids)) { igraph_vector_destroy(&v_vids); }

  return 0;
}

/*-------------------------------------------/
/ igraph_edge_betweenness                    /
/-------------------------------------------*/
void shell_igraph_edge_betweenness_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_edge_betweenness(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t res;
  igraph_bool_t directed=1;

  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "directed",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));
  shell_seen[2]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_vector_init(&res, 0);
      break;
    case 2: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 3:
      shell_igraph_edge_betweenness_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_edge_betweenness(&graph, &res, directed, 0);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&res, shell_arg_res); 
  igraph_vector_destroy(&res);

  return 0;
}

/*-------------------------------------------/
/ igraph_edge_betweenness_estimate           /
/-------------------------------------------*/
void shell_igraph_edge_betweenness_estimate_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --directed=<directed> --cutoff=<cutoff>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_edge_betweenness_estimate(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t res;
  igraph_bool_t directed=1;
  igraph_real_t cutoff;

  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "directed",required_argument,0,2 },
                                   { "cutoff",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[2]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_vector_init(&res, 0);
      break;
    case 2: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 3: /* cutoff */
      shell_read_real(&cutoff, optarg);
      break;
    case 4:
      shell_igraph_edge_betweenness_estimate_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_edge_betweenness_estimate(&graph, &res, directed, cutoff, 0);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&res, shell_arg_res); 
  igraph_vector_destroy(&res);

  return 0;
}

/*-------------------------------------------/
/ igraph_pagerank_old                        /
/-------------------------------------------*/
void shell_igraph_pagerank_old_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --vids=<vids> --directed=<directed> --niter=<niter> --eps=<eps> --damping=<damping> --old=<old>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_pagerank_old(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t res;
  igraph_vector_t v_vids; igraph_vs_t vids=igraph_vss_all();
  igraph_bool_t directed=1;
  igraph_integer_t niter=1000;
  igraph_real_t eps=0.001;
  igraph_real_t damping=0.85;
  igraph_bool_t old=0;
  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[8];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "vids",required_argument,0,2 },
                                   { "directed",required_argument,0,3 },
                                   { "niter",required_argument,0,4 },
                                   { "eps",required_argument,0,5 },
                                   { "damping",required_argument,0,6 },
                                   { "old",required_argument,0,7 },
                                   { "help",no_argument,0,8 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 8*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  shell_seen[4]=2;
  shell_seen[5]=2;
  shell_seen[6]=2;
  shell_seen[7]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_vector_init(&res, 0);
      break;
    case 2: /* vids */
      shell_read_vector(&v_vids, optarg); igraph_vs_vector(&vids, &v_vids);
      break;
    case 3: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 4: /* niter */
      shell_read_integer(&niter, optarg);
      break;
    case 5: /* eps */
      shell_read_real(&eps, optarg);
      break;
    case 6: /* damping */
      shell_read_real(&damping, optarg);
      break;
    case 7: /* old */
      shell_read_boolean(&old, optarg);
      break;
    case 8:
      shell_igraph_pagerank_old_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<8; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_pagerank_old(&graph, &res, vids, directed, niter, eps, damping, old);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&res, shell_arg_res); 
  igraph_vector_destroy(&res);
  if (!igraph_vs_is_all(&vids)) { igraph_vector_destroy(&v_vids); }

  return 0;
}

/*-------------------------------------------/
/ igraph_pagerank                            /
/-------------------------------------------*/
void shell_igraph_pagerank_usage(char **argv) {
  printf("%s --graph=<graph> --vector=<vector> --value=<value> --vids=<vids> --directed=<directed> --damping=<damping>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_pagerank(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t vector;
  igraph_real_t value;
  igraph_vector_t v_vids; igraph_vs_t vids=igraph_vss_all();
  igraph_bool_t directed=1;
  igraph_real_t damping=0.85;


  char* shell_arg_vector=0;
  char* shell_arg_value=0;
  char* shell_arg_options=0;
  int shell_result;


  int shell_seen[6];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "vector",required_argument,0,1 },
                                   { "value",required_argument,0,2 },
                                   { "vids",required_argument,0,3 },
                                   { "directed",required_argument,0,4 },
                                   { "damping",required_argument,0,5 },
                                   { "help",no_argument,0,6 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 6*sizeof(int));
  shell_seen[3]=2;
  shell_seen[4]=2;
  shell_seen[5]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* vector */
      shell_arg_vector=strdup(optarg); 
  igraph_vector_init(&vector, 0);
      break;
    case 2: /* value */
      shell_arg_value=strdup(optarg);
      break;
    case 3: /* vids */
      shell_read_vector(&v_vids, optarg); igraph_vs_vector(&vids, &v_vids);
      break;
    case 4: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 5: /* damping */
      shell_read_real(&damping, optarg);
      break;
    case 6:
      shell_igraph_pagerank_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<6; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_pagerank(&graph, &vector, &value, vids, directed, damping, 0, 0);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&vector, shell_arg_vector); 
  igraph_vector_destroy(&vector);
  shell_write_real(value, shell_arg_value);
  if (!igraph_vs_is_all(&vids)) { igraph_vector_destroy(&v_vids); }

  return 0;
}

/*-------------------------------------------/
/ igraph_rewire                              /
/-------------------------------------------*/
void shell_igraph_rewire_usage(char **argv) {
  printf("%s --rewire=<rewire> --rewire-out=<rewire-out> --n=<n> --mode=<mode>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_rewire(int argc, char **argv) {

  igraph_t rewire;
  igraph_integer_t n;
  igraph_rewiring_t mode=IGRAPH_REWIRING_SIMPLE;
  char* shell_arg_rewire=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "rewire",required_argument,0,0 },
                                   { "rewire-out",required_argument,0,1 },
                                   { "n",required_argument,0,2 },
                                   { "mode",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* rewire */
      shell_read_graph(&rewire, optarg);
      break;
    case 1: /* rewire-out */
      shell_arg_rewire=strdup(optarg);
      break;
    case 2: /* n */
      shell_read_integer(&n, optarg);
      break;
    case 3: /* mode */
      shell_read_enum(&mode, optarg, "simple", 0, 0);
      break;
    case 4:
      shell_igraph_rewire_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_rewire(&rewire, n, mode);

  /* Write the result */
  igraph_destroy(&rewire);
  shell_write_graph(&rewire, shell_arg_rewire); 
  igraph_destroy(&rewire);

  return 0;
}

/*-------------------------------------------/
/ igraph_subgraph                            /
/-------------------------------------------*/
void shell_igraph_subgraph_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --vids=<vids>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_subgraph(int argc, char **argv) {

  igraph_t graph;
  igraph_t res;
  igraph_vector_t v_vids; igraph_vs_t vids;
  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[3];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "vids",required_argument,0,2 },
                                   { "help",no_argument,0,3 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 3*sizeof(int));

  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg);
      break;
    case 2: /* vids */
      shell_read_vector(&v_vids, optarg); igraph_vs_vector(&vids, &v_vids);
      break;
    case 3:
      shell_igraph_subgraph_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<3; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_subgraph(&graph, &res, vids);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_graph(&res, shell_arg_res); 
  igraph_destroy(&res);
  if (!igraph_vs_is_all(&vids)) { igraph_vector_destroy(&v_vids); }

  return 0;
}

/*-------------------------------------------/
/ igraph_average_path_length                 /
/-------------------------------------------*/
void shell_igraph_average_path_length_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --directed=<directed> --unconn=<unconn>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_average_path_length(int argc, char **argv) {

  igraph_t graph;
  igraph_real_t res;
  igraph_bool_t directed=1;
  igraph_bool_t unconn=1;
  char* shell_arg_res=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "directed",required_argument,0,2 },
                                   { "unconn",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg);
      break;
    case 2: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 3: /* unconn */
      shell_read_boolean(&unconn, optarg);
      break;
    case 4:
      shell_igraph_average_path_length_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_average_path_length(&graph, &res, directed, unconn);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_real(res, shell_arg_res);

  return 0;
}

/*-------------------------------------------/
/ igraph_path_length_hist                    /
/-------------------------------------------*/
void shell_igraph_path_length_hist_usage(char **argv) {
  printf("%s --graph=<graph> --res=<res> --unconnected=<unconnected> --directed=<directed>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_path_length_hist(int argc, char **argv) {

  igraph_t graph;
  igraph_vector_t res;
  igraph_real_t unconnected;
  igraph_bool_t directed=1;
  char* shell_arg_res=0;
  char* shell_arg_unconnected=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "res",required_argument,0,1 },
                                   { "unconnected",required_argument,0,2 },
                                   { "directed",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* res */
      shell_arg_res=strdup(optarg); 
  igraph_vector_init(&res, 0);
      break;
    case 2: /* unconnected */
      shell_arg_unconnected=strdup(optarg);
      break;
    case 3: /* directed */
      shell_read_boolean(&directed, optarg);
      break;
    case 4:
      shell_igraph_path_length_hist_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_path_length_hist(&graph, &res, &unconnected, directed);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_vector(&res, shell_arg_res); 
  igraph_vector_destroy(&res);
  shell_write_real(unconnected, shell_arg_unconnected);

  return 0;
}

/*-------------------------------------------/
/ igraph_simplify                            /
/-------------------------------------------*/
void shell_igraph_simplify_usage(char **argv) {
  printf("%s --graph=<graph> --graph-out=<graph-out> --multiple=<multiple> --loops=<loops>\n", basename(argv[0]));
  exit(1);
}

int shell_igraph_simplify(int argc, char **argv) {

  igraph_t graph;
  igraph_bool_t multiple=1;
  igraph_bool_t loops=1;
  char* shell_arg_graph=0;
  int shell_result;


  int shell_seen[4];
  int shell_index=-1;
  struct option shell_options[]= { { "graph",required_argument,0,0 },
                                   { "graph-out",required_argument,0,1 },
                                   { "multiple",required_argument,0,2 },
                                   { "loops",required_argument,0,3 },
                                   { "help",no_argument,0,4 },
                                   { 0,0,0,0 }
                                 };

  /* 0 - not seen, 1 - seen as argument, 2 - seen as default */
  memset(shell_seen, 0, 4*sizeof(int));
  shell_seen[2]=2;
  shell_seen[3]=2;
  
  /* Parse arguments and read input */
  while (getopt_long(argc, argv, "", shell_options, &shell_index) != -1) {

    if (shell_index==-1) {
      exit(1);
    }

    if (shell_seen[shell_index]==1) {
      fprintf(stderr, "Error, `--%s' argument given twice.\n",
              shell_options[shell_index].name);
      exit(1);
    }
    shell_seen[shell_index]=1;  

    switch (shell_index) {
    case 0: /* graph */
      shell_read_graph(&graph, optarg);
      break;
    case 1: /* graph-out */
      shell_arg_graph=strdup(optarg);
      break;
    case 2: /* multiple */
      shell_read_boolean(&multiple, optarg);
      break;
    case 3: /* loops */
      shell_read_boolean(&loops, optarg);
      break;
    case 4:
      shell_igraph_simplify_usage(argv);
      break;
    default:
      break;
    }

    shell_index=-1;
  }

  /* Check that we have all arguments */
  for (shell_index=0; shell_index<4; shell_index++) {
    if (!shell_seen[shell_index]) {
      fprintf(stderr, "Error, argument missing: `--%s'.\n",
              shell_options[shell_index].name);
      exit(1);
    }
  }

  /* Do the operation */
  shell_result=igraph_simplify(&graph, multiple, loops);

  /* Write the result */
  igraph_destroy(&graph);
  shell_write_graph(&graph, shell_arg_graph); 
  igraph_destroy(&graph);

  return 0;
}
